/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.abstractschema;

import org.tranql.schema.Association;
import org.tranql.schema.AssociationEnd;
import org.tranql.schema.Entity;



/**
 *
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public abstract class AbstractAssociation implements Association {
   private final Entity mtmEntity;
   private final AssociationEnd[] ends = new AssociationEnd[2];
   private final JoinDefinition[] joinDefinitions;
   
   public AbstractAssociation(JoinDefinition joinDefinition) {
       this.mtmEntity = null;
       joinDefinitions = new JoinDefinition[] {joinDefinition};
   }

   public AbstractAssociation(Entity mtmEntity, JoinDefinition leftDefinition, JoinDefinition rightDefinition) {
       this.mtmEntity = mtmEntity;
       joinDefinitions = new JoinDefinition[] {leftDefinition, rightDefinition};
   }
   
   public AssociationEnd[] getAssociationEnds() {
       checkState(ends);
       return ends;
   }

   public AssociationEnd getOtherEnd(AssociationEnd end) {
       checkState(ends);
       if ( ends[0] == end ) {
           return ends[1];
       } else if ( ends[1] == end ) {
           return ends[0];
       }
       throw new IllegalArgumentException(end + " is unknown.");
   }

   public void addAssociationEnd(AssociationEnd end) {
       if ( null == ends[0] ) {
           ends[0] = end;
           return;
       } else if ( null == ends[1] ) {
           ends[1] = end;
           return;
       }
       throw new IllegalStateException("Already two ends are defined.");
   }
   
   public Entity getManyToManyEntity() {
       return mtmEntity;
   }

   public JoinDefinition getJoinDefinition() {
       if ( 2 == joinDefinitions.length ) {
           throw new UnsupportedOperationException();
       }
       return joinDefinitions[0];
   }

   public JoinDefinition getLeftJoinDefinition() {
       if ( 1 == joinDefinitions.length ) {
           throw new UnsupportedOperationException();
       }
       return joinDefinitions[0];
   }
   
   public JoinDefinition getRightJoinDefinition() {
       if ( 1 == joinDefinitions.length ) {
           throw new UnsupportedOperationException();
       }
       return joinDefinitions[1];
   }
   
   private void checkState(Object[] opaque) {
       if ( null == opaque[0] ) {
           throw new IllegalStateException("No AssociationEnd are defined.");
       } else if ( null == opaque[1] ) {
           throw new IllegalStateException("Only one AssociationEnd is defined.");
       }
   }
}
