/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.abstractschema;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.tranql.schema.AssociationEnd;
import org.tranql.schema.Attribute;
import org.tranql.schema.Entity;

/**
 *
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class AbstractEntity implements Entity {
    private final String name;
    private final String physicalName;

    private final List attributes = new ArrayList();
    private final List pkFields = new ArrayList();
    private final Map attributeByName = new HashMap();

    private final List associationEnds = new ArrayList();
    private final Map associationEndByName = new HashMap();

    protected AbstractEntity(String name, String physicalName) {
        this.name = name;
        this.physicalName = physicalName;
    }
    
    public String getName() {
        return name;
    }

    public String getPhysicalName() {
        return physicalName;
    }

    public List getPrimaryKeyFields() {
        return Collections.unmodifiableList(pkFields);
    }

    protected void addAttribute(Attribute attribute) {
        attributes.add(attribute);
        attributeByName.put(attribute.getName(), attribute);
        if (attribute.isIdentity()) {
            pkFields.add(attribute);
        }
    }

    public List getAttributes() {
        return Collections.unmodifiableList(attributes);
    }

    public Attribute getAttribute(String name) {
        return (Attribute) attributeByName.get(name);
    }

    protected void addAssociationEnd(AssociationEnd associationEnd) {
        associationEnds.add(associationEnd);
        associationEndByName.put(associationEnd.getName(), associationEnd);
    }

    public AssociationEnd getAssociationEnd(String name) {
        return (AssociationEnd) associationEndByName.get(name);
    }

    public AssociationEnd getAssociationEndDefiningFKAttribute(String fkAttribute) {
        for (Iterator iter = associationEndByName.values().iterator(); iter.hasNext();) {
            AssociationEnd end = (AssociationEnd) iter.next();
            if (end.hasFKAttribute(fkAttribute)) {
                return end;
            }
        }
        return null;
    }
    
    public List getAssociationEnds() {
        return Collections.unmodifiableList(associationEnds);
    }

    public String toString() {
        return "Entity logical name=[" + name + "]; physical name=[" + physicalName + "]";
    }
}
