/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.abstractschema;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.tranql.query.CommandFactory;
import org.tranql.schema.Entity;
import org.tranql.schema.Schema;

/**
 *
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public abstract class AbstractSchema implements Schema {
    private final String name;
    private final String physicalName;
    private final CommandFactory commandFactory;
    protected final Map entities = new HashMap();
    
    public AbstractSchema(String name, String physicalName, CommandFactory commandFactory) {
        this.name = name;
        this.physicalName = physicalName;
        this.commandFactory = commandFactory;
    }

    public String getName() {
        return name;
    }

    public String getPhysicalName() {
        return physicalName;
    }

    public CommandFactory getCommandFactory() {
        return commandFactory;
    }

    public Entity getEntity(String name) {
        return (Entity) entities.get(name);
    }
    
    public Entity getEntityByPhysicalName(String physicalName) {
        for (Iterator iter = entities.values().iterator(); iter.hasNext();) {
            Entity entity = (Entity) iter.next();
            if (entity.getPhysicalName().equals(physicalName)) {
                return entity;
            }
        }
        throw new IllegalArgumentException("No entity having the physical name " + physicalName);
    }
    
    public Map getEntities() {
        return Collections.unmodifiableMap(entities);
    }
    
    protected void addEntity(Entity entity) {
        entities.put(entity.getName(), entity);
    }    
}
