/**
 *  Copyright 2006 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.builder;

import java.io.ObjectStreamException;

import org.tranql.cache.GlobalSchema;
import org.tranql.ejb.EJBSchema;
import org.tranql.ql.QueryException;
import org.tranql.query.CommandTransform;
import org.tranql.query.SchemaMapper;
import org.tranql.query.UpdateCommand;
import org.tranql.sql.SQLSchema;
import org.tranql.sql.UpdateCommandBuilder;

public class StaticCommandBuilder implements UpdateCommandBuilder {
    protected final String name;
    protected final EJBSchema ejbSchema;
    protected final SQLSchema sqlSchema;
    protected final GlobalSchema globalSchema;
    private transient  EJBQueryBuilder queryBuilder;
    private transient CommandTransform mapper;

    public StaticCommandBuilder(String name,
            EJBSchema ejbSchema,
            SQLSchema sqlSchema,
            GlobalSchema globalSchema) {
        this.name = name;
        this.ejbSchema = ejbSchema;
        this.sqlSchema = sqlSchema;
        this.globalSchema = globalSchema;
        
        queryBuilder = new EJBQueryBuilder(new IdentityDefinerBuilder(ejbSchema, globalSchema));
        mapper = new SchemaMapper(sqlSchema);
    }

    public UpdateCommand buildCreate() throws QueryException {
        UpdateCommand command = queryBuilder.buildCreate(name);
        return mapper.transform(command);
    }

    public UpdateCommand buildStore() throws QueryException {
        UpdateCommand command = queryBuilder.buildStore(name);
        return mapper.transform(command);
    }

    public UpdateCommand buildRemove() throws QueryException {
        UpdateCommand command = queryBuilder.buildRemove(name);
        return mapper.transform(command);
    }

    private Object readResolve() throws ObjectStreamException {
        return new StaticCommandBuilder(name, ejbSchema, sqlSchema, globalSchema);
    }
}
