/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.cache;

import org.tranql.field.FieldTransform;
import org.tranql.field.NullAccessor;
import org.tranql.ql.AttributeReference;
import org.tranql.ql.From;
import org.tranql.ql.Node;
import org.tranql.ql.Query;
import org.tranql.ql.QueryBinding;
import org.tranql.ql.QueryException;
import org.tranql.ql.QuerySource;
import org.tranql.ql.Select;
import org.tranql.query.QueryCloner;
import org.tranql.query.QueryCommand;
import org.tranql.query.UpdateCommand;
import org.tranql.schema.Attribute;
import org.tranql.schema.Entity;
import org.tranql.schema.Schema;

/**
 * 
 * 
 * @version $Revision: 155 $ $Date: 2004-12-02 18:12:10 -0500 (Thu, 02 Dec 2004) $
 */
public class CacheLoadTransform extends QueryCloner {
    private final Schema schema;
    private final Entity entity;
    private final Attribute[] attributes;
    private final FieldTransform[] cacheTransforms;

    public CacheLoadTransform(Schema schema, String entityName, String[] attributeNames) {
        this.schema = schema;
        entity = schema.getEntity(entityName);
        attributes = new Attribute[attributeNames.length];
        cacheTransforms = new FieldTransform[attributes.length];
        for (int i = 0; i < attributeNames.length; i++) {
            Attribute attribute = entity.getAttribute(attributeNames[i]);
            attributes[i] = attribute;
            cacheTransforms[i] = new NullAccessor(attribute.getType());
        }
    }

    private class Context {
        private QuerySource source;
    }

    public QueryCommand transform(QueryCommand source) throws QueryException {
        Query query = source.getQuery();
        Context ctx = new Context();
        Query cacheQuery = (Query) query.visit(this, ctx);
        return new CacheLoadCommand(schema.getCommandFactory().createQuery(cacheQuery), null, null);
    }

    public UpdateCommand transform(UpdateCommand source) throws QueryException {
        throw new UnsupportedOperationException("Cache load not supported for updates");
    }

    public Object visit(Query node, Object param) throws QueryException {
        QueryBinding[] resultTransforms = node.getResultTransforms();
        QueryBinding[] cacheResultTransforms = new QueryBinding[resultTransforms.length + attributes.length];
        System.arraycopy(resultTransforms, 0, cacheResultTransforms, 0, resultTransforms.length);
        System.arraycopy(cacheTransforms, 0, cacheResultTransforms, resultTransforms.length, attributes.length);
        Query to = new Query(node.getParamTransforms(), resultTransforms);

        // @todo restructure Query so that From gets visited before Select, then all this goes away
        Select select = (Select) node.getChild();
        From from = (From) select.getSibling();
        From newFrom = (From) from.visit(this, param);
        Select newSelect = (Select) select.visit(this, param);
        to.addChild(newSelect).addChild(newFrom);
        for (Node child = from.getSibling(); child != null; child = child.getSibling()) {
            to.addChild((Node) child.visit(this, param));
        }
        return super.visit(node, param);
    }

    public Object visit(QuerySource node, Object param) throws QueryException {
        QuerySource to = (QuerySource) super.visit(node, param);
        if (entity.equals(to.getEntity())) {
            Context ctx = (Context) param;
            ctx.source = to;
        }
        return to;
    }

    public Object visit(Select node, Object param) throws QueryException {
        Context ctx = (Context) param;
        Select to = (Select) super.visit(node, param);
        for (int i = 0; i < attributes.length; i++) {
            Attribute attribute = attributes[i];
            to.addChild(new AttributeReference(ctx.source, attribute));
        }
        return to;
    }
}
