/**
 *
 *  Copyright 2004 Dain Sundstrom, Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.cache;

import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityDefiner;
import org.tranql.ql.QueryException;

/**
 * @version $Revision: 61 $ $Date: 2004-04-22 23:27:07 -0400 (Thu, 22 Apr 2004) $
 */
public class NothingFaultHandler implements FaultHandler {
    private final CacheTable cacheTable;
    private final IdentityDefiner identityDefiner;

    public NothingFaultHandler(CacheTable cacheTable, IdentityDefiner identityDefiner) {
        this.cacheTable = cacheTable;
        this.identityDefiner = identityDefiner;
    }

    public void rowFault(InTxCache cache, GlobalIdentity globalId) throws QueryException {
        CacheRow cacheRow = cacheTable.emptyRow(globalId);
        fieldFault(cache, cacheRow);
    }

    public void fieldFault(InTxCache cache, CacheRow cacheRow) throws QueryException {
        identityDefiner.injectIdentity(cacheRow);
        try {
            cache.associate(cacheRow);
        } catch (AlreadyAssociatedException e) {
            throw new QueryException(e);
        }
    }
}
