/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.cache;

import org.tranql.field.Row;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityDefiner;
import org.tranql.ql.QueryException;
import org.tranql.query.QueryCommand;
import org.tranql.query.ResultHandler;

/**
 *
 *
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public final class QueryFaultHandler implements FaultHandler {
    private final QueryCommand command;
    private final IdentityDefiner idDefiner;
    private final EmptySlotLoader[] accessors;

    public QueryFaultHandler(QueryCommand command, IdentityDefiner idDefiner, EmptySlotLoader[] accessors) {
        this.command = command;
        this.idDefiner = idDefiner;
        this.accessors = accessors;
    }

    public void fieldFault(InTxCache cache, CacheRow row) throws QueryException {
        GlobalIdentity id = row.getId();
        Row params = idDefiner.extractIdentity(id);
        ResultHandler handler = new CacheRowLoader(accessors);
        command.execute(cache, handler, params, row);
    }

    public void rowFault(InTxCache cache, GlobalIdentity id) throws QueryException {
        CacheRow row = id.getTable().emptyRow(id);
        Row params = idDefiner.extractIdentity(id);
        CacheRowLoader rowLoader = new CacheRowLoader(accessors); 
        ResultHandler handler = rowLoader;
        command.execute(cache, handler, params, row);
        if (rowLoader.wasFetched()) {
            // query command may have already added the id to the cache.
            if (null == cache.get(id)) {
                try {
                    cache.associate(row);
                } catch (AlreadyAssociatedException e) {
                    throw (IllegalStateException) new IllegalStateException("Row already associated after fault").initCause(e);
                }
            }
        }
    }
}
