/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.cache;

import java.util.Iterator;
import java.util.Map;

import org.tranql.query.UpdateCommand;
import org.tranql.ql.QueryException;

/**
 * Very basic flush strategy that assumes tables are independent and
 * does not factor in dependencies caused by relationships.
 *
 * @version $Revision: 276 $ $Date: 2005-10-01 04:41:03 -0400 (Sat, 01 Oct 2005) $
 */
public class SimpleFlushStrategy implements CacheFlushStrategy {
    public void rowAssociated(CacheRow row) {
    }
    
    public void rowAdded(CacheRow row) {
    }

    public void rowModified(CacheRow row) {
    }

    public void rowRemoved(CacheRow row) {
    }

    public void flush(Map cache) throws QueryException {
        for (Iterator tables = cache.entrySet().iterator(); tables.hasNext();) {
            Map.Entry entry = (Map.Entry) tables.next();
            CacheTable table = (CacheTable) entry.getKey();
            Map tableMap = (Map) entry.getValue();
            flush(table, tableMap);
        }
    }

    private void flush(CacheTable table, Map tableMap) throws QueryException {
        UpdateCommand insertCommand = table.getInsertCommand();
        UpdateCommand updateCommand = table.getUpdateCommand();
        UpdateCommand deleteCommand = table.getDeleteCommand();

        for (Iterator rows = tableMap.values().iterator(); rows.hasNext();) {
            CacheRow row = (CacheRow) rows.next();
            CacheRowState state = row.getState();
            if (state == CacheRowState.NEW) {
                insertCommand.execute(null, row);
            } else if (state == CacheRowState.MODIFIED) {
                updateCommand.execute(null, row);
            } else if (state == CacheRowState.REMOVED) {
                deleteCommand.execute(null, row);
            }
            row.markClean();
        }
    }
}
