/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.cache;

import org.tranql.field.FieldTransform;
import org.tranql.field.Row;
import org.tranql.field.FieldTransformException;

/**
 *
 *
 *
 * @version $Revision: 233 $ $Date: 2005-08-19 23:48:40 -0400 (Fri, 19 Aug 2005) $
 */
public class UnderConstructionSlotDetector implements FieldTransform {
    private final FieldTransform delegate;
    private final int slot;

    public UnderConstructionSlotDetector(FieldTransform delegate, int slot) {
        this.delegate = delegate;
        this.slot = slot;
    }

    public Class getFieldClass() {
        return delegate.getFieldClass();
    }

    public Object get(Row row) throws FieldTransformException {
        checkState(row);
        return delegate.get(row);
    }

    public void set(Row row, Object value) throws FieldTransformException {
        checkState(row);
        delegate.set(row, value);
    }
    
    private void checkState(Row row) {
        CacheRow cacheRow = (CacheRow)row;
        if (cacheRow.isSlotUnderConstruction(slot)) {
            throw new IllegalStateException("Slot is under construction. " +
                    "A CMR mapped to a foreign key column, which is also a " +
                    "primary key column has not been set.");
        }
    }
}
