/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ddl;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;



/**
 *
 *
 * @version $Revision: 246 $ $Date: 2005-08-26 22:09:42 -0400 (Fri, 26 Aug 2005) $
 */
public class BaseDataTypeMappingRegistry implements DataTypeMappingRegistry {
    private static final Class DEFAULT_CLASS = BaseDataTypeMappingRegistry.class;
    private static final int DEFAULT_PRECISION = 10;
    private static final int DEFAULT_SCALE = 5;
    private static final int DEFAULT_LENGTH = 255;
    
    private final Map sqlTypeToFormat;
    private final Map classToFormat;
    private int defaultPrecision = DEFAULT_PRECISION;
    private int defaultScale = DEFAULT_SCALE;
    private int defaultLength = DEFAULT_LENGTH;
    
    public BaseDataTypeMappingRegistry() {
        sqlTypeToFormat = new HashMap();
        classToFormat = new HashMap();
    }
    
    public void register(int sqlType, String format) {
        sqlTypeToFormat.put(new Integer(sqlType), format);
    }
    
    public void register(Class clazz, String format) {
        classToFormat.put(clazz, format);
    }
    
    public void registerDefault(String format) {
        classToFormat.put(DEFAULT_CLASS, format);
    }
    
    public String getDataType(int sqlType, int precision, int scale, int length) {
        String format = (String) sqlTypeToFormat.get(new Integer(sqlType));
        if (null == format) {
            throw new IllegalArgumentException("SQL Type " + sqlType + " is not defined.");
        }
        return format(precision, scale, length, format);
    }
    
    public String getDataType(Class clazz, int precision, int scale, int length) {
        String format = (String) classToFormat.get(clazz);
        if (null == format) {
            format = (String) classToFormat.get(DEFAULT_CLASS);
            if (null == format) {
                throw new IllegalArgumentException("Class " + clazz + " is not defined. No default is defined.");
            }
        }
        return format(precision, scale, length, format);
    }

    public int getDefaultLength() {
        return defaultLength;
    }

    public void setDefaultLength(int defaultLength) {
        this.defaultLength = defaultLength;
    }

    public int getDefaultPrecision() {
        return defaultPrecision;
    }

    public void setDefaultPrecision(int defaultPrecision) {
        this.defaultPrecision = defaultPrecision;
    }

    public int getDefaultScale() {
        return defaultScale;
    }

    public void setDefaultScale(int defaultScale) {
        this.defaultScale = defaultScale;
    }
    
    private String format(int precision, int scale, int length, String format) {
        if (-1 == precision) {
            precision = defaultPrecision;
        }
        if (-1 == scale) {
            scale = defaultScale;
        }
        if (-1 == length) {
            length = defaultLength;
        }
        MessageFormat mf = new MessageFormat(format);
        Object[] objs = {new Integer(precision), new Integer(scale), new Integer(length)};
        return mf.format(objs);
    }
}