/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import java.util.Set;
import java.util.Collection;
import java.util.Iterator;

import org.tranql.cache.AlreadyAssociatedException;
import org.tranql.cache.CacheRow;
import org.tranql.cache.InTxCache;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityDefiner;

/**
 * 
 * 
 * @version $Revision: 127 $ $Date: 2004-10-14 21:52:57 -0400 (Thu, 14 Oct 2004) $
 */
public abstract class AbstractRelationSet implements Set {
    protected final Set idSet;

    public AbstractRelationSet(InTxCache cache, IdentityDefiner idDefiner, Set idSet) {
        this.idSet = idSet;
        for (Iterator iter = idSet.iterator(); iter.hasNext();) {
            GlobalIdentity id = (GlobalIdentity) iter.next();
            if ( null == cache.get(id) ) {
                CacheRow row = id.getTable().emptyRow(id);
                try {
                    cache.associate(row);
                } catch (AlreadyAssociatedException e) {
                    // This can not happen.
                    throw new AssertionError(e);
                }
                idDefiner.injectIdentity(row);
            }
        }
    }

    protected abstract void associate(GlobalIdentity b2);

    protected abstract void disassociate(GlobalIdentity b2);

    public boolean add(Object o) {
        associate((GlobalIdentity) o);
        return idSet.add(o);
    }

    public boolean addAll(Collection c) {
        for (Iterator i = c.iterator(); i.hasNext();) {
            GlobalIdentity b = (GlobalIdentity) i.next();
            associate(b);
        }
        return idSet.addAll(c);
    }

    public void clear() {
        for (Iterator i = idSet.iterator(); i.hasNext();) {
            GlobalIdentity globalIdentity = (GlobalIdentity) i.next();
            disassociate(globalIdentity);
        }
        idSet.clear();
    }

    public boolean remove(Object o) {
        disassociate((GlobalIdentity) o);
        return idSet.remove(o);
    }

    public boolean removeAll(Collection c) {
        for (Iterator i = c.iterator(); i.hasNext();) {
            GlobalIdentity globalIdentity = (GlobalIdentity) i.next();
            if (idSet.contains(globalIdentity)) {
                disassociate(globalIdentity);
            }
        }
        return idSet.removeAll(c);
    }

    public boolean retainAll(Collection c) {
        for (Iterator i = idSet.iterator(); i.hasNext();) {
            GlobalIdentity globalIdentity = (GlobalIdentity) i.next();
            if (c.contains(globalIdentity)) {
                disassociate(globalIdentity);
            }
        }
        return idSet.retainAll(c);
    }

    public Object[] toArray() {
        return idSet.toArray();
    }

    public Object[] toArray(Object a[]) {
        return idSet.toArray(a);
    }

    public boolean containsAll(Collection c) {
        return idSet.containsAll(c);
    }

    public boolean equals(Object o) {
        return idSet.equals(o);
    }

    public int hashCode() {
        return idSet.hashCode();
    }

    public int size() {
        return idSet.size();
    }

    public boolean isEmpty() {
        return idSet.isEmpty();
    }

    public boolean contains(Object o) {
        return idSet.contains(o);
    }
}
