/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import javax.ejb.EJBException;

import org.tranql.cache.CacheRow;
import org.tranql.cache.InTxCache;
import org.tranql.field.FieldTransform;
import org.tranql.field.FieldTransformException;

/**
 *
 *
 *
 * @version $Revision: 45 $ $Date: 2004-04-19 18:23:20 -0400 (Mon, 19 Apr 2004) $
 */
public class CMPFieldAccessor implements CMPFieldTransform {
    private final FieldTransform transform;
    private final String fieldName;

    public CMPFieldAccessor(FieldTransform transform, String fieldName) {
        this.transform = transform;
        this.fieldName = fieldName;
    }

    public Object get(InTxCache cache, CacheRow row) {
        try {
            Object value = transform.get(row);
            if (value == CacheRow.NO_DATA) {
                throw new NotLoadedException("CMP-Field is not loaded " + fieldName);
            }
            return value;
        } catch (FieldTransformException e) {
            throw new EJBException("Unable to fetch value for cmp-field " + fieldName, e);
        }
    }

    public void set(InTxCache cache, CacheRow row, Object value) {
        try {
            transform.set(row, value);
        } catch (FieldTransformException e) {
            throw new EJBException("Unable to set value for cmp-field " + fieldName, e);
        }
        cache.modified(row);
    }
}
