/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import org.tranql.cache.CacheRow;
import org.tranql.cache.CacheRowState;
import org.tranql.cache.InTxCache;
import org.tranql.identity.GlobalIdentity;

/**
 *
 * @version $Revision: 233 $ $Date: 2005-08-19 23:48:40 -0400 (Fri, 19 Aug 2005) $
 */
public class CMRMappedToInversePKCMP implements CMPFieldTransform {
    private final CMPFieldTransform cmrField;
    private final int cmrSlot;
    
    public CMRMappedToInversePKCMP(CMPFieldTransform cmrField, int cmrSlot) {
        this.cmrField = cmrField;
        this.cmrSlot = cmrSlot;
    }

    public Object get(InTxCache cache, CacheRow row) {
        return cmrField.get(cache, row);
    }

    public void set(InTxCache cache, CacheRow row, Object value) {
        GlobalIdentity id = (GlobalIdentity) value;
        if (CacheRowState.REMOVED == row.getState()) {
            cmrField.set(cache, row, value);
            return;
        }
        if (null == id) {
            throw new IllegalStateException("Trying to change a primary key by resetting the " +
                "inverse side of a relationship.");
        }
        CacheRow relatedRow = cache.get(id);
        if (null == relatedRow) {
            throw new IllegalStateException("Trying to change a primary key by resetting the " +
                "inverse side of a relationship.");
        }
        if (relatedRow.isSlotUnderConstruction(cmrSlot)) {
            GlobalIdentity expectedID = (GlobalIdentity) relatedRow.get(cmrSlot);
            if (false == expectedID.equals(row.getId())) {
                throw new IllegalStateException("Trying to change a primary key by setting the " +
                        "inverse side of a relationship.");
            }
            relatedRow.unmarkSlotUnderConstruction(cmrSlot);
        }
        cmrField.set(cache, row, value);
    }
}