/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import java.util.Arrays;



/**
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public final class FinderEJBQLQuery implements EJBQLQuery {
    private static final Class[] NOARGS = new Class[0];
    
    private final String methodName;
    private final Class[] parameterTypes;
    private final String ejbQL;
    private boolean flushCacheBeforeQuery = true;
    private final int hashCode;
    private String prefetchGroup;

    public FinderEJBQLQuery(String methodName, Class[] parameterTypes, String ejbQL) {
        this.methodName = methodName;
        if (null == parameterTypes) {
            this.parameterTypes = NOARGS;
        } else {
            this.parameterTypes = parameterTypes;
        }
        this.ejbQL = ejbQL;
        
        int hashCodeTmp = methodName.hashCode();
        for (int i = 0; i < this.parameterTypes.length; i++) {
            hashCodeTmp += this.parameterTypes[i].hashCode();
        }
        hashCode = hashCodeTmp;
    }

    public String getMethodName() {
        return methodName;
    }

    public Class[] getParameterTypes() {
        return parameterTypes;
    }

    public String getEjbQL() {
        return ejbQL;
    }
    
    public void setFlushCacheBeforeQuery(boolean flushCacheBeforeQuery) {
        this.flushCacheBeforeQuery = flushCacheBeforeQuery;
    }
    
    public boolean isFlushCacheBeforeQuery() {
        return flushCacheBeforeQuery;
    }

    public void setPrefetchGroup(String prefetchGroup) {
        this.prefetchGroup = prefetchGroup;
    }
    
    public String getPrefetchGroup() {
        return prefetchGroup;
    }

    public String toString() {
        return "Finder method=[" + methodName + "]; EJB-QL=[" + ejbQL + "]";
    }
    
    public int hashCode() {
        return hashCode;
    }
    
    public boolean equals(Object obj) {
        if (false == obj instanceof FinderEJBQLQuery) {
            return false;
        }
        FinderEJBQLQuery other = (FinderEJBQLQuery) obj;
        return methodName.equals(other.methodName) && Arrays.equals(parameterTypes, other.parameterTypes);
    }
}
