/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import java.rmi.RemoteException;
import javax.ejb.EJBObject;

import org.tranql.field.FieldTransform;
import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;

/**
 * Field transform that converts a primary key into a EJBObject.
 *
 * @version $Revision: 57 $ $Date: 2004-04-21 17:45:17 -0400 (Wed, 21 Apr 2004) $
 */
public class IdAsEJBObjectTransform implements FieldTransform {
    private final FieldTransform next;
    private final EJBProxyFactory factory;
    private final Class fieldClass;

    public IdAsEJBObjectTransform(FieldTransform next, EJBProxyFactory factory, Class pkClass) {
        this.next = next;
        this.factory = factory;
        this.fieldClass = pkClass;
    }

    public Class getFieldClass() {
        return fieldClass;
    }

    public Object get(Row row) throws FieldTransformException {
        try {
            EJBObject proxy = (EJBObject) next.get(row);
            return proxy.getPrimaryKey();
        } catch (RemoteException e) {
            throw new FieldTransformException("Unable to fetch primary key for reference", e);
        }
    }

    public void set(Row row, Object value) throws FieldTransformException {
        next.set(row, factory.getEJBObject(value));
    }
}
