/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import java.util.HashSet;
import java.util.Set;

import org.tranql.cache.CacheRow;
import org.tranql.cache.FaultHandler;
import org.tranql.cache.InTxCache;
import org.tranql.field.FieldTransform;
import org.tranql.field.Row;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityDefiner;
import org.tranql.ql.QueryException;
import org.tranql.query.CollectionResultHandler;
import org.tranql.query.QueryCommand;
import org.tranql.query.ResultHandler;

/**
 *
 *
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class MultiValuedCMRFaultHandler implements FaultHandler {
    private final QueryCommand command;
    private final int slot;
    private final IdentityDefiner idDefiner;
    private final CollectionResultHandler collectionHandler;

    public MultiValuedCMRFaultHandler(QueryCommand command, int slot, IdentityDefiner idDefiner, FieldTransform globalIDDefiner) {
        this.command = command;
        this.slot = slot;
        this.idDefiner = idDefiner;
        collectionHandler = new CollectionResultHandler(globalIDDefiner);
    }

    public void fieldFault(InTxCache cache, CacheRow row) throws QueryException {
        Row params = idDefiner.extractIdentity(row.getId());
        Set results = new HashSet();
        ResultHandler handler = collectionHandler;
        command.execute(cache, handler, params, results);
        row.set(slot, results);
    }

    public void rowFault(InTxCache cache, GlobalIdentity id) throws QueryException {
        throw new UnsupportedOperationException();
    }
}
