/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import java.util.Set;

import org.tranql.cache.AlreadyAssociatedException;
import org.tranql.cache.CacheRow;
import org.tranql.cache.InTxCache;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityDefiner;

/**
 *
 *
 *
 * @version $Revision: 159 $ $Date: 2005-01-07 01:22:20 -0500 (Fri, 07 Jan 2005) $
 */
public class OneToManyCMR implements CMPFieldTransform {
    private final CMPFieldTransform next;
    private final CMPFieldTransform related;
    private final IdentityDefiner relatedIDDefiner;

    public OneToManyCMR(CMPFieldTransform next, CMPFieldTransform related, IdentityDefiner relatedIDDefiner) {
        this.next = next;
        this.related = related;
        this.relatedIDDefiner = relatedIDDefiner;
    }

    public Object get(InTxCache cache, CacheRow row) {
        return next.get(cache, row);
    }

    public void set(InTxCache cache, CacheRow a1row, Object value) {
        GlobalIdentity a1 = a1row.getId();
        GlobalIdentity b2 = (GlobalIdentity) value;
        GlobalIdentity b1 = (GlobalIdentity) next.get(cache, a1row);
        if (b1 != null) {
            CacheRow b1Row = cache.get(b1);
            if ( null == b1Row ) {
                b1Row = b1.getTable().emptyRow(b1);
                try {
                    cache.associate(b1Row);
                } catch (AlreadyAssociatedException e) {
                    throw new AssertionError(e);
                }
                relatedIDDefiner.injectIdentity(b1Row);
            }
            Set relatedSet = (Set) related.get(cache, b1Row);
            relatedSet.remove(a1);
        }
        if (b2 != null) {
            CacheRow b2row = cache.get(b2);
            if ( null == b2row ) {
                b2row = b2.getTable().emptyRow(b2);
                try {
                    cache.associate(b2row);
                } catch (AlreadyAssociatedException e) {
                    throw new AssertionError(e);
                }
                relatedIDDefiner.injectIdentity(b2row);
            }
            Set relatedSet = (Set) related.get(cache, b2row);
            relatedSet.add(a1);
        }
        next.set(cache, a1row, b2);
    }
}
