/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import org.tranql.cache.AlreadyAssociatedException;
import org.tranql.cache.CacheRow;
import org.tranql.cache.InTxCache;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityDefiner;

/**
 *
 *
 *
 * @version $Revision: 144 $ $Date: 2004-11-09 10:55:34 -0500 (Tue, 09 Nov 2004) $
 */
public class OneToOneCMR implements CMPFieldTransform {
    private final CMPFieldTransform next;
    private final IdentityDefiner nextIDDefiner;
    private final CMPFieldTransform related;
    private final IdentityDefiner relatedIDDefiner;

    public OneToOneCMR(CMPFieldTransform next, IdentityDefiner nextIDDefiner, CMPFieldTransform related, IdentityDefiner relatedIDDefiner) {
        this.next = next;
        this.nextIDDefiner = nextIDDefiner;
        this.related = related;
        this.relatedIDDefiner = relatedIDDefiner;
    }

    public Object get(InTxCache cache, CacheRow row) {
        return next.get(cache, row);
    }

    public void set(InTxCache cache, CacheRow a1row, Object value) {
        GlobalIdentity a1 = a1row.getId();
        GlobalIdentity b2 = (GlobalIdentity) value;
        GlobalIdentity b1 = (GlobalIdentity) next.get(cache, a1row);
        if (b1 != null) {
            CacheRow b1Row = cache.get(b1);
            if ( null == b1Row ) {
                b1Row = b1.getTable().emptyRow(b1);
                try {
                    cache.associate(b1Row);
                } catch (AlreadyAssociatedException e) {
                    throw new AssertionError(e);
                }
                relatedIDDefiner.injectIdentity(b1Row);
            }
            related.set(cache, b1Row, null);
        }
        if (b2 != null) {
            CacheRow b2row = cache.get(b2);
            if ( null == b2row ) {
                b2row = b2.getTable().emptyRow(b2);
                try {
                    cache.associate(b2row);
                } catch (AlreadyAssociatedException e) {
                    throw new AssertionError(e);
                }
                relatedIDDefiner.injectIdentity(b2row);
            }
            GlobalIdentity a2 = (GlobalIdentity) related.get(cache, b2row);
            if (a2 != null) {
                CacheRow a2row = cache.get(a2);
                if ( null == a2row ) {
                    a2row = a2.getTable().emptyRow(a2);
                    try {
                        cache.associate(a2row);
                    } catch (AlreadyAssociatedException e) {
                        // This can not happen.
                        throw new AssertionError(e);
                    }
                    nextIDDefiner.injectIdentity(a2row);
                }
                next.set(cache, a2row, null);
            }
            related.set(cache, b2row, a1);
        }
        next.set(cache, a1row, b2);
    }
}
