/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import java.rmi.RemoteException;
import javax.ejb.EJBException;
import javax.ejb.EJBObject;

import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityTransform;
import org.tranql.identity.IdentityTransformException;

/**
 * Identity transform that converts between an EJBObject reference
 * and a row reference into the cache.
 *
 * @version $Revision: 57 $ $Date: 2004-04-21 17:45:17 -0400 (Wed, 21 Apr 2004) $
 */
public class RemoteProxyTransform implements IdentityTransform {
    private final IdentityTransform next;
    private final EJBProxyFactory factory;

    public RemoteProxyTransform(IdentityTransform next, EJBProxyFactory factory) {
        this.next = next;
        this.factory = factory;
    }

    public GlobalIdentity getGlobalIdentity(Object domainId) throws IdentityTransformException {
        try {
            Object pk = ((EJBObject) domainId).getPrimaryKey();
            return next.getGlobalIdentity(pk);
        } catch (RemoteException e) {
            throw new EJBException("Unable to getPrimaryKey() from proxy", e);
        }
    }

    public Object getDomainIdentity(GlobalIdentity globalId) throws IdentityTransformException {
        Object pk = next.getDomainIdentity(globalId);
        return factory.getEJBObject(pk);
    }
}
