/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb;

import javax.ejb.EJBException;

import org.tranql.cache.CacheRow;
import org.tranql.cache.CacheRowState;
import org.tranql.cache.InTxCache;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityTransform;
import org.tranql.identity.IdentityTransformException;

/**
 *
 *
 *
 * @version $Revision: 156 $ $Date: 2004-12-05 05:00:27 -0500 (Sun, 05 Dec 2004) $
 */
public class SingleValuedCMRAccessor implements CMPFieldTransform {
    private final CMPFieldTransform next;
    private final IdentityTransform idTransform;

    public SingleValuedCMRAccessor(CMPFieldTransform next, IdentityTransform idTransform) {
        this.next = next;
        this.idTransform = idTransform;
    }

    public Object get(InTxCache cache, CacheRow row) {
        GlobalIdentity globalId = (GlobalIdentity) next.get(cache, row);
        try {
            return globalId == null ? null : idTransform.getDomainIdentity(globalId);
        } catch (IdentityTransformException e) {
            throw new EJBException("Unable to create proxy for id " + globalId, e);
        }
    }

    public void set(InTxCache cache, CacheRow row, Object value) {
        if (value != null) {
            try {
                value = idTransform.getGlobalIdentity(value);
            } catch (IdentityTransformException e) {
                throw new EJBException("Unable to extract id from proxy " + value, e);
            }

            // Check that the object to be assigned to this CMR has not been removed.
            CacheRow cacheRow = cache.get((GlobalIdentity) value);
            if (null != cacheRow && CacheRowState.REMOVED == cacheRow.getState()) {
                throw new IllegalArgumentException("[EJB2.1 10.3.4.1] Attempt to assign a removed entity object as the value of a cmr-field of another object.");
            }
        }
        next.set(cache, row, value);
    }
}
