// $ANTLR 2.7.4: "ejbql21.g" -> "EJBQL21Compiler.java"$

/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejb.parser;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import antlr.TreeParser;
import antlr.Token;
import antlr.collections.AST;
import antlr.RecognitionException;
import antlr.ANTLRException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.collections.impl.BitSet;
import antlr.ASTPair;
import antlr.collections.impl.ASTArray;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Collections;
import java.util.ArrayList;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.tranql.field.FieldTransform;
import org.tranql.schema.AssociationEnd;
import org.tranql.schema.Association;
import org.tranql.schema.Attribute;
import org.tranql.schema.Entity;
import org.tranql.schema.Schema;
import org.tranql.ql.*;


public class EJBQL21Compiler extends antlr.TreeParser       implements EJBQLTokenTypes
 {

    private static final Log log = LogFactory.getLog(EJBQL21Compiler.class);
    private boolean error = false;
    private QueryBinding[] paramTransforms;
    private Schema schema;
    private final Map aliases = new HashMap();

    public EJBQL21Compiler(QueryBinding[] paramTransforms, Schema schema) {
        this();
        this.paramTransforms = paramTransforms;
        this.schema = schema;
    }

    public void reportError(RecognitionException e) {
        error = true;
        log.error("", e);
    }

    public void reportError(String s) {
        error = true;
        log.error(s);
    }

    public void reportWarning(String s) {
        log.warn(s);
    }

    public boolean hadErrors() {
        return error;
    }

    public AliasedEntity getAlias(String alias) {
        return (AliasedEntity) aliases.get(alias);
    }

    public void addAlias(AliasedEntity aliasedEntity) {
        aliases.put(aliasedEntity.getAlias(), aliasedEntity);
    }
    
public EJBQL21Compiler() {
	tokenNames = _tokenNames;
}

	public final Query  ejbql(AST _t) throws RecognitionException {
		Query query;
		
		AST ejbql_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		From from;
		Select select;
		
		
		AST __t89 = _t;
		AST tmp1_AST_in = (AST)_t;
		match(_t,EJBQL);
		_t = _t.getFirstChild();
		query = new Query(paramTransforms, new QueryBinding[0]);
		{
		from=from_clause(_t);
		_t = _retTree;
		select=select_clause(_t);
		_t = _retTree;
		}
		query.addChild(select); query.addChild(from);
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case WHERE:
		{
			Where where;
			where=where_clause(_t);
			_t = _retTree;
			query.addChild(where);
			break;
		}
		case 3:
		case ORDER:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case ORDER:
		{
			OrderBy orderBy;
			orderBy=orderby_clause(_t);
			_t = _retTree;
			query.addChild(orderBy);
			break;
		}
		case 3:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		_t = __t89;
		_t = _t.getNextSibling();
		_retTree = _t;
		return query;
	}
	
	public final From  from_clause(AST _t) throws RecognitionException {
		From from = new From();
		
		AST from_clause_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node lastNode = null;
		Node currentNode = null;
		
		
		
		AST __t94 = _t;
		AST tmp2_AST_in = (AST)_t;
		match(_t,FROM);
		_t = _t.getFirstChild();
		{
		int _cnt97=0;
		_loop97:
		do {
			if (_t==null) _t=ASTNULL;
			if ((_t.getType()==RANGE_VARIABLE||_t.getType()==COLLECTION_MEMBER)) {
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case RANGE_VARIABLE:
				{
					currentNode=range_variable_declaration(_t);
					_t = _retTree;
					break;
				}
				case COLLECTION_MEMBER:
				{
					currentNode=collection_member_declaration(_t);
					_t = _retTree;
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				
				if (lastNode == null) {
				lastNode = currentNode;
				} else {
				Join join = new Join(Join.CROSS_JOIN);
				lastNode = join.addChild(lastNode).addChild(currentNode);
				}
				
			}
			else {
				if ( _cnt97>=1 ) { break _loop97; } else {throw new NoViableAltException(_t);}
			}
			
			_cnt97++;
		} while (true);
		}
		_t = __t94;
		_t = _t.getNextSibling();
		
		from.addChild(lastNode);
		
		_retTree = _t;
		return from;
	}
	
	public final Select  select_clause(AST _t) throws RecognitionException {
		Select select;
		
		AST select_clause_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		boolean distinct = false;
		
		
		AST __t126 = _t;
		AST tmp3_AST_in = (AST)_t;
		match(_t,SELECT);
		_t = _t.getFirstChild();
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case DISTINCT:
		{
			AST tmp4_AST_in = (AST)_t;
			match(_t,DISTINCT);
			_t = _t.getNextSibling();
			distinct = true;
			break;
		}
		case PATH:
		case IDENTIFIER:
		case AVG:
		case MIN:
		case MAX:
		case SUM:
		case COUNT:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		select = new Select(distinct);
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case PATH:
		{
			Path path;
			path=path_expression(_t);
			_t = _retTree;
			select.addChild(path);
			break;
		}
		case AVG:
		case MIN:
		case MAX:
		case SUM:
		case COUNT:
		{
			AggregateFunction function;
			function=aggregate_expression(_t);
			_t = _retTree;
			select.addChild(function);
			break;
		}
		case IDENTIFIER:
		{
			AST tmp5_AST_in = (AST)_t;
			match(_t,IDENTIFIER);
			_t = _t.getNextSibling();
			
			String alias = tmp5_AST_in.getText();
			AliasedEntity entity = (AliasedEntity) aliases.get(alias);
			if (entity == null) {
			throw new SemanticException("No alias exists with name " + alias);
			}
			select.addChild(new EntityReference(entity));
			
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		_t = __t126;
		_t = _t.getNextSibling();
		_retTree = _t;
		return select;
	}
	
	public final Where  where_clause(AST _t) throws RecognitionException {
		Where where;
		
		AST where_clause_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node expression;
		
		
		AST __t130 = _t;
		AST tmp6_AST_in = (AST)_t;
		match(_t,WHERE);
		_t = _t.getFirstChild();
		expression=boolean_expression(_t);
		_t = _retTree;
		_t = __t130;
		_t = _t.getNextSibling();
		
		where = new Where();
		where.addChild(expression);
		
		_retTree = _t;
		return where;
	}
	
	public final OrderBy  orderby_clause(AST _t) throws RecognitionException {
		OrderBy orderBy;
		
		AST orderby_clause_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		boolean isAsc = true;
		List paths = new ArrayList();
		Path path;
		
		
		AST __t132 = _t;
		AST tmp7_AST_in = (AST)_t;
		match(_t,ORDER);
		_t = _t.getFirstChild();
		{
		int _cnt134=0;
		_loop134:
		do {
			if (_t==null) _t=ASTNULL;
			if ((_t.getType()==PATH)) {
				path=path_expression(_t);
				_t = _retTree;
				paths.add(path);
			}
			else {
				if ( _cnt134>=1 ) { break _loop134; } else {throw new NoViableAltException(_t);}
			}
			
			_cnt134++;
		} while (true);
		}
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case ASC:
		{
			AST tmp8_AST_in = (AST)_t;
			match(_t,ASC);
			_t = _t.getNextSibling();
			isAsc = true;
			break;
		}
		case DESC:
		{
			AST tmp9_AST_in = (AST)_t;
			match(_t,DESC);
			_t = _t.getNextSibling();
			isAsc = false;
			break;
		}
		case 3:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		
		orderBy = new OrderBy(isAsc);
		for (int i = 0; i < paths.size(); i++) {
		orderBy.addChild((Node) paths.get(i));
		}
		
		_t = __t132;
		_t = _t.getNextSibling();
		_retTree = _t;
		return orderBy;
	}
	
	public final QuerySource  range_variable_declaration(AST _t) throws RecognitionException {
		QuerySource source;
		
		AST range_variable_declaration_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		String abstractSchemaName;
		String alias;
		
		
		AST __t99 = _t;
		AST tmp10_AST_in = (AST)_t;
		match(_t,RANGE_VARIABLE);
		_t = _t.getFirstChild();
		abstractSchemaName=abstract_schema_name(_t);
		_t = _retTree;
		alias=identifier(_t);
		_t = _retTree;
		_t = __t99;
		_t = _t.getNextSibling();
		
		Entity entity = schema.getEntityByPhysicalName(abstractSchemaName);
		if (entity == null) {
		throw new SemanticException("No EJB having abstract schema name " + abstractSchemaName);
		}
		source = new QuerySource(entity, alias);
		addAlias(source);
		
		_retTree = _t;
		return source;
	}
	
	public final DerivedTable  collection_member_declaration(AST _t) throws RecognitionException {
		DerivedTable derivedTable;
		
		AST collection_member_declaration_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Path path;
		String alias;
		
		
		AST __t101 = _t;
		AST tmp11_AST_in = (AST)_t;
		match(_t,COLLECTION_MEMBER);
		_t = _t.getFirstChild();
		path=path_expression(_t);
		_t = _retTree;
		alias=identifier(_t);
		_t = _retTree;
		_t = __t101;
		_t = _t.getNextSibling();
		
		derivedTable = new DerivedTable(path, alias);
		addAlias(derivedTable);
		
		_retTree = _t;
		return derivedTable;
	}
	
	public final String  abstract_schema_name(AST _t) throws RecognitionException {
		String name;
		
		AST abstract_schema_name_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST schema = null;
		
		schema = (AST)_t;
		match(_t,ABSTRACT_SCHEMA_NAME);
		_t = _t.getNextSibling();
		name = schema.getText();
		_retTree = _t;
		return name;
	}
	
	public final String  identifier(AST _t) throws RecognitionException {
		String name;
		
		AST identifier_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST id = null;
		
		id = (AST)_t;
		match(_t,IDENTIFIER);
		_t = _t.getNextSibling();
		name = id.getText();
		_retTree = _t;
		return name;
	}
	
	public final Path  path_expression(AST _t) throws RecognitionException {
		Path path;
		
		AST path_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		String root;
		String part;
		String fullPath;
		
		AliasedEntity rootEntity;
		Entity currentEntity;
		List pathList = new LinkedList();
		
		
		AST __t105 = _t;
		AST tmp12_AST_in = (AST)_t;
		match(_t,PATH);
		_t = _t.getFirstChild();
		root=identifier(_t);
		_t = _retTree;
		
		rootEntity = getAlias(root);
		currentEntity = rootEntity.getEntity();
		fullPath = root;
		
		{
		int _cnt107=0;
		_loop107:
		do {
			if (_t==null) _t=ASTNULL;
			if ((_t.getType()==IDENTIFIER)) {
				part=identifier(_t);
				_t = _retTree;
				
				if (currentEntity == null) {
				throw new SemanticException("A cmp-field path can not be further navigated: " + fullPath);
				}
				AssociationEnd endpoint = currentEntity.getAssociationEnd(part);
				if (endpoint != null) {
				pathList.add(endpoint);
				Association association = endpoint.getAssociation();
				currentEntity = endpoint.getEntity();
				} else {
				Attribute attribute = currentEntity.getAttribute(part);
				if (attribute == null) {
				throw new SemanticException("Entity " + currentEntity.getName() + " does not contain a CMP or CMR field named " + part);
				}
				pathList.add(attribute);
				currentEntity = null;
				}
				fullPath += "." + part;
				
			}
			else {
				if ( _cnt107>=1 ) { break _loop107; } else {throw new NoViableAltException(_t);}
			}
			
			_cnt107++;
		} while (true);
		}
		_t = __t105;
		_t = _t.getNextSibling();
		
		path = new Path(rootEntity, pathList);
		
		_retTree = _t;
		return path;
	}
	
	public final AggregateFunction  aggregate_expression(AST _t) throws RecognitionException {
		AggregateFunction function;
		
		AST aggregate_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case AVG:
		{
			function=avg_aggregate_expression(_t);
			_t = _retTree;
			break;
		}
		case MIN:
		{
			function=min_aggregate_expression(_t);
			_t = _retTree;
			break;
		}
		case MAX:
		{
			function=max_aggregate_expression(_t);
			_t = _retTree;
			break;
		}
		case SUM:
		{
			function=sum_aggregate_expression(_t);
			_t = _retTree;
			break;
		}
		case COUNT:
		{
			function=count_aggregate_expression(_t);
			_t = _retTree;
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		_retTree = _t;
		return function;
	}
	
	public final AggregateFunction  avg_aggregate_expression(AST _t) throws RecognitionException {
		AggregateFunction function;
		
		AST avg_aggregate_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Path path;
		boolean distinct = false;
		
		
		AST __t110 = _t;
		AST tmp13_AST_in = (AST)_t;
		match(_t,AVG);
		_t = _t.getFirstChild();
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case DISTINCT:
		{
			AST tmp14_AST_in = (AST)_t;
			match(_t,DISTINCT);
			_t = _t.getNextSibling();
			distinct=true;
			break;
		}
		case PATH:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		path=path_expression(_t);
		_t = _retTree;
		_t = __t110;
		_t = _t.getNextSibling();
		
		function = new AggregateFunction(AggregateFunction.AVG, distinct);
		function.addChild(path);
		
		_retTree = _t;
		return function;
	}
	
	public final AggregateFunction  min_aggregate_expression(AST _t) throws RecognitionException {
		AggregateFunction function;
		
		AST min_aggregate_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Path path;
		boolean distinct = false;
		
		
		AST __t113 = _t;
		AST tmp15_AST_in = (AST)_t;
		match(_t,MIN);
		_t = _t.getFirstChild();
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case DISTINCT:
		{
			AST tmp16_AST_in = (AST)_t;
			match(_t,DISTINCT);
			_t = _t.getNextSibling();
			distinct=true;
			break;
		}
		case PATH:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		path=path_expression(_t);
		_t = _retTree;
		_t = __t113;
		_t = _t.getNextSibling();
		
		function = new AggregateFunction(AggregateFunction.MIN, distinct);
		function.addChild(path);
		
		_retTree = _t;
		return function;
	}
	
	public final AggregateFunction  max_aggregate_expression(AST _t) throws RecognitionException {
		AggregateFunction function;
		
		AST max_aggregate_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Path path;
		boolean distinct = false;
		
		
		AST __t116 = _t;
		AST tmp17_AST_in = (AST)_t;
		match(_t,MAX);
		_t = _t.getFirstChild();
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case DISTINCT:
		{
			AST tmp18_AST_in = (AST)_t;
			match(_t,DISTINCT);
			_t = _t.getNextSibling();
			distinct=true;
			break;
		}
		case PATH:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		path=path_expression(_t);
		_t = _retTree;
		_t = __t116;
		_t = _t.getNextSibling();
		
		function = new AggregateFunction(AggregateFunction.MAX, distinct);
		function.addChild(path);
		
		_retTree = _t;
		return function;
	}
	
	public final AggregateFunction  sum_aggregate_expression(AST _t) throws RecognitionException {
		AggregateFunction function;
		
		AST sum_aggregate_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Path path;
		boolean distinct = false;
		
		
		AST __t119 = _t;
		AST tmp19_AST_in = (AST)_t;
		match(_t,SUM);
		_t = _t.getFirstChild();
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case DISTINCT:
		{
			AST tmp20_AST_in = (AST)_t;
			match(_t,DISTINCT);
			_t = _t.getNextSibling();
			distinct=true;
			break;
		}
		case PATH:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		path=path_expression(_t);
		_t = _retTree;
		_t = __t119;
		_t = _t.getNextSibling();
		
		function = new AggregateFunction(AggregateFunction.SUM, distinct);
		function.addChild(path);
		
		_retTree = _t;
		return function;
	}
	
	public final AggregateFunction  count_aggregate_expression(AST _t) throws RecognitionException {
		AggregateFunction function;
		
		AST count_aggregate_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		boolean distinct = false;
		
		
		AST __t122 = _t;
		AST tmp21_AST_in = (AST)_t;
		match(_t,COUNT);
		_t = _t.getFirstChild();
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case DISTINCT:
		{
			AST tmp22_AST_in = (AST)_t;
			match(_t,DISTINCT);
			_t = _t.getNextSibling();
			distinct=true;
			break;
		}
		case PATH:
		case IDENTIFIER:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		function = new CountAggregateFunction(distinct);
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case PATH:
		{
			Path path;
			path=path_expression(_t);
			_t = _retTree;
			function.addChild(path);
			break;
		}
		case IDENTIFIER:
		{
			AST tmp23_AST_in = (AST)_t;
			match(_t,IDENTIFIER);
			_t = _t.getNextSibling();
			
			String alias = tmp23_AST_in.getText();
			AliasedEntity entity = (AliasedEntity) aliases.get(alias);
			if (entity == null) {
			throw new SemanticException("No alias exists with name " + alias);
			}
			function.addChild(new EntityReference(entity));
			
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		_t = __t122;
		_t = _t.getNextSibling();
		_retTree = _t;
		return function;
	}
	
	public final Node  boolean_expression(AST _t) throws RecognitionException {
		Node node;
		
		AST boolean_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case EXPLICIT_BRACKETING:
		{
			node=explicit_bracketing_boolean_expression(_t);
			_t = _retTree;
			break;
		}
		case AND:
		{
			node=and_expression(_t);
			_t = _retTree;
			break;
		}
		case OR:
		{
			node=or_expression(_t);
			_t = _retTree;
			break;
		}
		case NOT:
		{
			node=not_expression(_t);
			_t = _retTree;
			break;
		}
		case EQ:
		case NE:
		case LT:
		case LE:
		case GT:
		case GE:
		{
			node=comparison_expression(_t);
			_t = _retTree;
			break;
		}
		case BETWEEN:
		{
			node=between_expression(_t);
			_t = _retTree;
			break;
		}
		case IN:
		{
			node=in_expression(_t);
			_t = _retTree;
			break;
		}
		case LIKE:
		{
			node=like_expression(_t);
			_t = _retTree;
			break;
		}
		case EMPTY:
		{
			node=empty_expression(_t);
			_t = _retTree;
			break;
		}
		case NULL:
		{
			node=is_null_expression(_t);
			_t = _retTree;
			break;
		}
		case MEMBER:
		{
			node=member_of_expression(_t);
			_t = _retTree;
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		_retTree = _t;
		return node;
	}
	
	public final RowConstructor  explicit_bracketing_boolean_expression(AST _t) throws RecognitionException {
		RowConstructor rowConstructor;
		
		AST explicit_bracketing_boolean_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node content;
		
		
		AST __t138 = _t;
		AST tmp24_AST_in = (AST)_t;
		match(_t,EXPLICIT_BRACKETING);
		_t = _t.getFirstChild();
		content=boolean_expression(_t);
		_t = _retTree;
		_t = __t138;
		_t = _t.getNextSibling();
		
		rowConstructor = new RowConstructor();
		rowConstructor.addChild(content);
		
		_retTree = _t;
		return rowConstructor;
	}
	
	public final BinaryOperation  and_expression(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST and_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t142 = _t;
		AST tmp25_AST_in = (AST)_t;
		match(_t,AND);
		_t = _t.getFirstChild();
		left=boolean_expression(_t);
		_t = _retTree;
		right=boolean_expression(_t);
		_t = _retTree;
		_t = __t142;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.AND);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final BinaryOperation  or_expression(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST or_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t140 = _t;
		AST tmp26_AST_in = (AST)_t;
		match(_t,OR);
		_t = _t.getFirstChild();
		left=boolean_expression(_t);
		_t = _retTree;
		right=boolean_expression(_t);
		_t = _retTree;
		_t = __t140;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.OR);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final Not  not_expression(AST _t) throws RecognitionException {
		Not not;
		
		AST not_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node expression;
		
		
		AST __t144 = _t;
		AST tmp27_AST_in = (AST)_t;
		match(_t,NOT);
		_t = _t.getFirstChild();
		expression=boolean_expression(_t);
		_t = _retTree;
		_t = __t144;
		_t = _t.getNextSibling();
		
		not = new Not();
		not.addChild(expression);
		
		_retTree = _t;
		return not;
	}
	
	public final BinaryOperation  comparison_expression(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST comparison_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case EQ:
		{
			binaryOperation=equals_comparison(_t);
			_t = _retTree;
			break;
		}
		case NE:
		{
			binaryOperation=not_equals_comparison(_t);
			_t = _retTree;
			break;
		}
		case LT:
		{
			binaryOperation=less_than_comparison(_t);
			_t = _retTree;
			break;
		}
		case LE:
		{
			binaryOperation=less_than_equals_comparison(_t);
			_t = _retTree;
			break;
		}
		case GT:
		{
			binaryOperation=greater_than_comparison(_t);
			_t = _retTree;
			break;
		}
		case GE:
		{
			binaryOperation=greater_than_equals_comparison(_t);
			_t = _retTree;
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		_retTree = _t;
		return binaryOperation;
	}
	
	public final Between  between_expression(AST _t) throws RecognitionException {
		Between between;
		
		AST between_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		boolean not = false;
		Node start;
		Node end;
		
		
		AST __t159 = _t;
		AST tmp28_AST_in = (AST)_t;
		match(_t,BETWEEN);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case NOT:
		{
			AST tmp29_AST_in = (AST)_t;
			match(_t,NOT);
			_t = _t.getNextSibling();
			not=true;
			break;
		}
		case PATH:
		case IDENTIFIER:
		case PLUS:
		case MINUS:
		case MULT:
		case DIV:
		case INPUT_PARAM:
		case STRING_LITERAL:
		case CONCAT:
		case SUBSTRING:
		case LENGTH:
		case LOCATE:
		case ABS:
		case SQRT:
		case MOD:
		case INTEGER_LITERAL:
		case FLOAT_LITERAL:
		case TRUE:
		case FALSE:
		case EXPLICIT_BRACKETING:
		case UNARY_PLUS:
		case UNARY_MINUS:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		start=expression(_t);
		_t = _retTree;
		end=expression(_t);
		_t = _retTree;
		_t = __t159;
		_t = _t.getNextSibling();
		
		between = new Between(not);
		between.addChild(left).addChild(start).addChild(end);
		
		_retTree = _t;
		return between;
	}
	
	public final In  in_expression(AST _t) throws RecognitionException {
		In in;
		
		AST in_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		boolean not = false;
		Node item;
		
		
		AST __t162 = _t;
		AST tmp30_AST_in = (AST)_t;
		match(_t,IN);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case NOT:
		{
			AST tmp31_AST_in = (AST)_t;
			match(_t,NOT);
			_t = _t.getNextSibling();
			not=true;
			break;
		}
		case INPUT_PARAM:
		case STRING_LITERAL:
		case INTEGER_LITERAL:
		case FLOAT_LITERAL:
		case TRUE:
		case FALSE:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		
		in = new In(not);
		in.addChild(left);
		
		{
		int _cnt166=0;
		_loop166:
		do {
			if (_t==null) _t=ASTNULL;
			if ((_tokenSet_0.member(_t.getType()))) {
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case STRING_LITERAL:
				case INTEGER_LITERAL:
				case FLOAT_LITERAL:
				case TRUE:
				case FALSE:
				{
					item=literal(_t);
					_t = _retTree;
					break;
				}
				case INPUT_PARAM:
				{
					item=input_parameter(_t);
					_t = _retTree;
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				in.addChild(item);
			}
			else {
				if ( _cnt166>=1 ) { break _loop166; } else {throw new NoViableAltException(_t);}
			}
			
			_cnt166++;
		} while (true);
		}
		_t = __t162;
		_t = _t.getNextSibling();
		_retTree = _t;
		return in;
	}
	
	public final Like  like_expression(AST _t) throws RecognitionException {
		Like like;
		
		AST like_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		boolean not = false;
		Node pattern;
		Node escape = null;
		
		
		AST __t168 = _t;
		AST tmp32_AST_in = (AST)_t;
		match(_t,LIKE);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case NOT:
		{
			AST tmp33_AST_in = (AST)_t;
			match(_t,NOT);
			_t = _t.getNextSibling();
			not=true;
			break;
		}
		case INPUT_PARAM:
		case STRING_LITERAL:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case STRING_LITERAL:
		{
			pattern=string_literal(_t);
			_t = _retTree;
			break;
		}
		case INPUT_PARAM:
		{
			pattern=input_parameter(_t);
			_t = _retTree;
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case INPUT_PARAM:
		case STRING_LITERAL:
		{
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case STRING_LITERAL:
			{
				escape=string_literal(_t);
				_t = _retTree;
				break;
			}
			case INPUT_PARAM:
			{
				escape=input_parameter(_t);
				_t = _retTree;
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			break;
		}
		case 3:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		_t = __t168;
		_t = _t.getNextSibling();
		
		like = new Like(not);
		like.addChild(left).addChild(pattern);
		if (escape != null) {
		like.addChild(escape);
		}
		
		_retTree = _t;
		return like;
	}
	
	public final Empty  empty_expression(AST _t) throws RecognitionException {
		Empty empty;
		
		AST empty_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node value;
		boolean not = false;
		
		
		AST __t174 = _t;
		AST tmp34_AST_in = (AST)_t;
		match(_t,EMPTY);
		_t = _t.getFirstChild();
		value=expression(_t);
		_t = _retTree;
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case NOT:
		{
			AST tmp35_AST_in = (AST)_t;
			match(_t,NOT);
			_t = _t.getNextSibling();
			not = true;
			break;
		}
		case 3:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		_t = __t174;
		_t = _t.getNextSibling();
		
		empty = new Empty(not);
		empty.addChild(value);
		
		_retTree = _t;
		return empty;
	}
	
	public final IsNull  is_null_expression(AST _t) throws RecognitionException {
		IsNull isNull;
		
		AST is_null_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node value;
		boolean not = false;
		
		
		AST __t177 = _t;
		AST tmp36_AST_in = (AST)_t;
		match(_t,NULL);
		_t = _t.getFirstChild();
		value=expression(_t);
		_t = _retTree;
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case NOT:
		{
			AST tmp37_AST_in = (AST)_t;
			match(_t,NOT);
			_t = _t.getNextSibling();
			not=true;
			break;
		}
		case 3:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		_t = __t177;
		_t = _t.getNextSibling();
		
		isNull = new IsNull(not);
		isNull.addChild(value);
		
		_retTree = _t;
		return isNull;
	}
	
	public final MemberOf  member_of_expression(AST _t) throws RecognitionException {
		MemberOf memberOf;
		
		AST member_of_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		boolean not = false;
		Node right;
		
		
		AST __t180 = _t;
		AST tmp38_AST_in = (AST)_t;
		match(_t,MEMBER);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case NOT:
		{
			AST tmp39_AST_in = (AST)_t;
			match(_t,NOT);
			_t = _t.getNextSibling();
			not = true;
			break;
		}
		case PATH:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		right=path_expression(_t);
		_t = _retTree;
		_t = __t180;
		_t = _t.getNextSibling();
		
		memberOf = new MemberOf(not);
		memberOf.addChild(left).addChild(right);
		
		_retTree = _t;
		return memberOf;
	}
	
	public final BinaryOperation  equals_comparison(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST equals_comparison_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t147 = _t;
		AST tmp40_AST_in = (AST)_t;
		match(_t,EQ);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t147;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.EQ);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final BinaryOperation  not_equals_comparison(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST not_equals_comparison_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t149 = _t;
		AST tmp41_AST_in = (AST)_t;
		match(_t,NE);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t149;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.NE);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final BinaryOperation  less_than_comparison(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST less_than_comparison_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t151 = _t;
		AST tmp42_AST_in = (AST)_t;
		match(_t,LT);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t151;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.LT);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final BinaryOperation  less_than_equals_comparison(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST less_than_equals_comparison_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t153 = _t;
		AST tmp43_AST_in = (AST)_t;
		match(_t,LE);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t153;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.LE);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final BinaryOperation  greater_than_comparison(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST greater_than_comparison_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t155 = _t;
		AST tmp44_AST_in = (AST)_t;
		match(_t,GT);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t155;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.GT);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final BinaryOperation  greater_than_equals_comparison(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST greater_than_equals_comparison_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t157 = _t;
		AST tmp45_AST_in = (AST)_t;
		match(_t,GE);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t157;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.GE);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final Node  expression(AST _t) throws RecognitionException {
		Node node;
		
		AST expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case EXPLICIT_BRACKETING:
		{
			node=explicit_bracketing_expression(_t);
			_t = _retTree;
			break;
		}
		case PLUS:
		case MINUS:
		case MULT:
		case DIV:
		{
			node=arithmetic_expression(_t);
			_t = _retTree;
			break;
		}
		case CONCAT:
		case SUBSTRING:
		case LENGTH:
		case LOCATE:
		case ABS:
		case SQRT:
		case MOD:
		{
			node=function_expression(_t);
			_t = _retTree;
			break;
		}
		case PATH:
		{
			node=path_expression(_t);
			_t = _retTree;
			break;
		}
		case STRING_LITERAL:
		case INTEGER_LITERAL:
		case FLOAT_LITERAL:
		case TRUE:
		case FALSE:
		{
			node=literal(_t);
			_t = _retTree;
			break;
		}
		case INPUT_PARAM:
		{
			node=input_parameter(_t);
			_t = _retTree;
			break;
		}
		case IDENTIFIER:
		{
			node=entity_reference(_t);
			_t = _retTree;
			break;
		}
		case UNARY_PLUS:
		{
			node=unary_plus(_t);
			_t = _retTree;
			break;
		}
		case UNARY_MINUS:
		{
			node=unary_minus(_t);
			_t = _retTree;
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		_retTree = _t;
		return node;
	}
	
	public final Literal  literal(AST _t) throws RecognitionException {
		Literal literal;
		
		AST literal_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case STRING_LITERAL:
		{
			literal=string_literal(_t);
			_t = _retTree;
			break;
		}
		case INTEGER_LITERAL:
		case FLOAT_LITERAL:
		{
			literal=numeric_literal(_t);
			_t = _retTree;
			break;
		}
		case TRUE:
		case FALSE:
		{
			literal=boolean_literal(_t);
			_t = _retTree;
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		_retTree = _t;
		return literal;
	}
	
	public final ParameterReference  input_parameter(AST _t) throws RecognitionException {
		ParameterReference parameterReference;
		
		AST input_parameter_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST p = null;
		
		p = (AST)_t;
		match(_t,INPUT_PARAM);
		_t = _t.getNextSibling();
		
		String value = p.getText();
		value = value.substring(1, value.length());
		parameterReference = new ParameterReference(Integer.parseInt(value)-1);
		
		_retTree = _t;
		return parameterReference;
	}
	
	public final StringLiteral  string_literal(AST _t) throws RecognitionException {
		StringLiteral stringLiteral;
		
		AST string_literal_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST s = null;
		
		s = (AST)_t;
		match(_t,STRING_LITERAL);
		_t = _t.getNextSibling();
		
		String string = s.getText();
		char[] value = string.toCharArray();
		if (value[0] != '\'') {
		throw new SemanticException("String literal must start with a single quote");
		}
		if (value[value.length-1] != '\'') {
		throw new SemanticException("String literal must end with a single quote");
		}
		StringBuffer buffer = new StringBuffer(value.length * 2);
		for (int i = 1; i < value.length-1; i++) {
		char c = value[i];
		if (c == '\'') {
		if (value[i+1] != '\'') {
		throw new SemanticException("A single quote in a string literal must be excaped with another single quote");
		}
		i++;
		}
		buffer.append(c);
		}
		stringLiteral = new StringLiteral(buffer.toString());
		
		_retTree = _t;
		return stringLiteral;
	}
	
	public final RowConstructor  explicit_bracketing_expression(AST _t) throws RecognitionException {
		RowConstructor rowConstructor;
		
		AST explicit_bracketing_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node content;
		
		
		AST __t184 = _t;
		AST tmp46_AST_in = (AST)_t;
		match(_t,EXPLICIT_BRACKETING);
		_t = _t.getFirstChild();
		content=expression(_t);
		_t = _retTree;
		_t = __t184;
		_t = _t.getNextSibling();
		
		rowConstructor = new RowConstructor();
		rowConstructor.addChild(content);
		
		_retTree = _t;
		return rowConstructor;
	}
	
	public final BinaryOperation  arithmetic_expression(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST arithmetic_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case PLUS:
		{
			binaryOperation=addition_expression(_t);
			_t = _retTree;
			break;
		}
		case MINUS:
		{
			binaryOperation=subtraction_expression(_t);
			_t = _retTree;
			break;
		}
		case MULT:
		{
			binaryOperation=multiplication_expression(_t);
			_t = _retTree;
			break;
		}
		case DIV:
		{
			binaryOperation=division_expression(_t);
			_t = _retTree;
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		_retTree = _t;
		return binaryOperation;
	}
	
	public final Function  function_expression(AST _t) throws RecognitionException {
		Function function;
		
		AST function_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case CONCAT:
		{
			function=concat_function(_t);
			_t = _retTree;
			break;
		}
		case SUBSTRING:
		{
			function=substring_function(_t);
			_t = _retTree;
			break;
		}
		case LENGTH:
		{
			function=length_function(_t);
			_t = _retTree;
			break;
		}
		case LOCATE:
		{
			function=locate_function(_t);
			_t = _retTree;
			break;
		}
		case ABS:
		{
			function=abs_function(_t);
			_t = _retTree;
			break;
		}
		case SQRT:
		{
			function=sqrt_function(_t);
			_t = _retTree;
			break;
		}
		case MOD:
		{
			function=mod_function(_t);
			_t = _retTree;
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		_retTree = _t;
		return function;
	}
	
	public final EntityReference  entity_reference(AST _t) throws RecognitionException {
		EntityReference entityReference;
		
		AST entity_reference_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		AST tmp47_AST_in = (AST)_t;
		match(_t,IDENTIFIER);
		_t = _t.getNextSibling();
		
		String alias = tmp47_AST_in.getText();
		AliasedEntity entity = (AliasedEntity) aliases.get(alias);
		if (entity == null) {
		throw new SemanticException("No alias exists with name " + alias);
		}
		entityReference = new EntityReference(entity);
		
		_retTree = _t;
		return entityReference;
	}
	
	public final UnaryOperator  unary_plus(AST _t) throws RecognitionException {
		UnaryOperator unaryOperator;
		
		AST unary_plus_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node content;
		
		
		AST __t186 = _t;
		AST tmp48_AST_in = (AST)_t;
		match(_t,UNARY_PLUS);
		_t = _t.getFirstChild();
		content=expression(_t);
		_t = _retTree;
		_t = __t186;
		_t = _t.getNextSibling();
		
			    unaryOperator = new UnaryOperator(UnaryOperator.PLUS);
			    unaryOperator.addChild(content);
			
		_retTree = _t;
		return unaryOperator;
	}
	
	public final UnaryOperator  unary_minus(AST _t) throws RecognitionException {
		UnaryOperator unaryOperator;
		
		AST unary_minus_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node content;
		
		
		AST __t188 = _t;
		AST tmp49_AST_in = (AST)_t;
		match(_t,UNARY_MINUS);
		_t = _t.getFirstChild();
		content=expression(_t);
		_t = _retTree;
		_t = __t188;
		_t = _t.getNextSibling();
		
			    unaryOperator = new UnaryOperator(UnaryOperator.MINUS);
			    unaryOperator.addChild(content);
			
		_retTree = _t;
		return unaryOperator;
	}
	
	public final BinaryOperation  addition_expression(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST addition_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t191 = _t;
		AST tmp50_AST_in = (AST)_t;
		match(_t,PLUS);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t191;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.PLUS);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final BinaryOperation  subtraction_expression(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST subtraction_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t193 = _t;
		AST tmp51_AST_in = (AST)_t;
		match(_t,MINUS);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t193;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.MINUS);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final BinaryOperation  multiplication_expression(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST multiplication_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t195 = _t;
		AST tmp52_AST_in = (AST)_t;
		match(_t,MULT);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t195;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.TIMES);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final BinaryOperation  division_expression(AST _t) throws RecognitionException {
		BinaryOperation binaryOperation;
		
		AST division_expression_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t197 = _t;
		AST tmp53_AST_in = (AST)_t;
		match(_t,DIV);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t197;
		_t = _t.getNextSibling();
		
		binaryOperation = new BinaryOperation(BinaryOperation.DIV);
		binaryOperation.addChild(left).addChild(right);
		
		_retTree = _t;
		return binaryOperation;
	}
	
	public final Function  concat_function(AST _t) throws RecognitionException {
		Function function;
		
		AST concat_function_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t200 = _t;
		AST tmp54_AST_in = (AST)_t;
		match(_t,CONCAT);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t200;
		_t = _t.getNextSibling();
		
		function = new Function(Function.CONCAT);
		function.addChild(left).addChild(right);
		
		_retTree = _t;
		return function;
	}
	
	public final Function  substring_function(AST _t) throws RecognitionException {
		Function function;
		
		AST substring_function_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node string;
		Node start;
		Node length;
		
		
		AST __t202 = _t;
		AST tmp55_AST_in = (AST)_t;
		match(_t,SUBSTRING);
		_t = _t.getFirstChild();
		string=expression(_t);
		_t = _retTree;
		start=expression(_t);
		_t = _retTree;
		length=expression(_t);
		_t = _retTree;
		_t = __t202;
		_t = _t.getNextSibling();
		
		function = new Function(Function.SUBSTRING);
		function.addChild(string).addChild(start).addChild(length);
		
		_retTree = _t;
		return function;
	}
	
	public final Function  length_function(AST _t) throws RecognitionException {
		Function function;
		
		AST length_function_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node string;
		
		
		AST __t204 = _t;
		AST tmp56_AST_in = (AST)_t;
		match(_t,LENGTH);
		_t = _t.getFirstChild();
		string=expression(_t);
		_t = _retTree;
		_t = __t204;
		_t = _t.getNextSibling();
		
		function = new Function(Function.LENGTH);
		function.addChild(string);
		
		_retTree = _t;
		return function;
	}
	
	public final Function  locate_function(AST _t) throws RecognitionException {
		Function function;
		
		AST locate_function_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node stringToLocate;
		Node source;
		Node start = null;
		
		
		AST __t206 = _t;
		AST tmp57_AST_in = (AST)_t;
		match(_t,LOCATE);
		_t = _t.getFirstChild();
		stringToLocate=expression(_t);
		_t = _retTree;
		source=expression(_t);
		_t = _retTree;
		{
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case PATH:
		case IDENTIFIER:
		case PLUS:
		case MINUS:
		case MULT:
		case DIV:
		case INPUT_PARAM:
		case STRING_LITERAL:
		case CONCAT:
		case SUBSTRING:
		case LENGTH:
		case LOCATE:
		case ABS:
		case SQRT:
		case MOD:
		case INTEGER_LITERAL:
		case FLOAT_LITERAL:
		case TRUE:
		case FALSE:
		case EXPLICIT_BRACKETING:
		case UNARY_PLUS:
		case UNARY_MINUS:
		{
			start=expression(_t);
			_t = _retTree;
			break;
		}
		case 3:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		}
		_t = __t206;
		_t = _t.getNextSibling();
		
		function = new Function(Function.LOCATE);
		function.addChild(stringToLocate).addChild(source);
		if (start != null) {
		function.addChild(start);
		}
		
		_retTree = _t;
		return function;
	}
	
	public final Function  abs_function(AST _t) throws RecognitionException {
		Function function;
		
		AST abs_function_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node number;
		
		
		AST __t209 = _t;
		AST tmp58_AST_in = (AST)_t;
		match(_t,ABS);
		_t = _t.getFirstChild();
		number=expression(_t);
		_t = _retTree;
		_t = __t209;
		_t = _t.getNextSibling();
		
		function = new Function(Function.ABS);
		function.addChild(number);
		
		_retTree = _t;
		return function;
	}
	
	public final Function  sqrt_function(AST _t) throws RecognitionException {
		Function function;
		
		AST sqrt_function_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node number;
		
		
		AST __t211 = _t;
		AST tmp59_AST_in = (AST)_t;
		match(_t,SQRT);
		_t = _t.getFirstChild();
		number=expression(_t);
		_t = _retTree;
		_t = __t211;
		_t = _t.getNextSibling();
		
		function = new Function(Function.SQRT);
		function.addChild(number);
		
		_retTree = _t;
		return function;
	}
	
	public final Function  mod_function(AST _t) throws RecognitionException {
		Function function;
		
		AST mod_function_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		Node left;
		Node right;
		
		
		AST __t213 = _t;
		AST tmp60_AST_in = (AST)_t;
		match(_t,MOD);
		_t = _t.getFirstChild();
		left=expression(_t);
		_t = _retTree;
		right=expression(_t);
		_t = _retTree;
		_t = __t213;
		_t = _t.getNextSibling();
		
		function = new Function(Function.MOD);
		function.addChild(left).addChild(right);
		
		_retTree = _t;
		return function;
	}
	
	public final NumericLiteral  numeric_literal(AST _t) throws RecognitionException {
		NumericLiteral numericLiteral;
		
		AST numeric_literal_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST i = null;
		AST f = null;
		
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case INTEGER_LITERAL:
		{
			i = (AST)_t;
			match(_t,INTEGER_LITERAL);
			_t = _t.getNextSibling();
			
			String value = i.getText();
			
			char lastChar = value.charAt(value.length()-1);
			if (lastChar == 'l' || lastChar == 'L') {
			// we need to strip off the 'l' or 'L'
			value = value.substring(0, value.length()-1);
			}
			
			if (value.startsWith("0x")) {
			value = value.substring(2, value.length());
			numericLiteral = new NumericLiteral(new BigInteger(value, 16), i.getText());
			} else if (value.startsWith("0")) {
			numericLiteral = new NumericLiteral(new BigInteger(value, 8), i.getText());
			} else {
			numericLiteral = new NumericLiteral(new BigInteger(value, 10), i.getText());
			}
			
			break;
		}
		case FLOAT_LITERAL:
		{
			f = (AST)_t;
			match(_t,FLOAT_LITERAL);
			_t = _t.getNextSibling();
			
			String value = f.getText();
			char lastChar = value.charAt(value.length()-1);
			if (!Character.isDigit(lastChar)) {
			// we need to strip off the 'f', 'F', 'd', 'D'
			value = value.substring(0, value.length()-1);
			}
			numericLiteral = new NumericLiteral(new BigDecimal(value), f.getText());
			
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		_retTree = _t;
		return numericLiteral;
	}
	
	public final BooleanLiteral  boolean_literal(AST _t) throws RecognitionException {
		BooleanLiteral booleanLiteral;
		
		AST boolean_literal_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		if (_t==null) _t=ASTNULL;
		switch ( _t.getType()) {
		case TRUE:
		{
			AST tmp61_AST_in = (AST)_t;
			match(_t,TRUE);
			_t = _t.getNextSibling();
			booleanLiteral = new BooleanLiteral(true);
			break;
		}
		case FALSE:
		{
			AST tmp62_AST_in = (AST)_t;
			match(_t,FALSE);
			_t = _t.getNextSibling();
			booleanLiteral = new BooleanLiteral(false);
			break;
		}
		default:
		{
			throw new NoViableAltException(_t);
		}
		}
		_retTree = _t;
		return booleanLiteral;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"EJBQL",
		"RANGE_VARIABLE",
		"COLLECTION_MEMBER",
		"ABSTRACT_SCHEMA_NAME",
		"PATH",
		"\"from\"",
		"COMMA",
		"\"in\"",
		"LPAREN",
		"\"as\"",
		"an identifier",
		"RPAREN",
		"\"select\"",
		"\"distinct\"",
		"\"object\"",
		"\"avg\"",
		"\"min\"",
		"\"max\"",
		"\"sum\"",
		"\"count\"",
		"\"order\"",
		"\"by\"",
		"\"asc\"",
		"\"desc\"",
		"\"where\"",
		"\"or\"",
		"\"and\"",
		"\"not\"",
		"EQ",
		"NE",
		"LT",
		"LE",
		"GT",
		"GE",
		"PLUS",
		"MINUS",
		"MULT",
		"DIV",
		"\"between\"",
		"an input parameter",
		"\"like\"",
		"a string literal",
		"\"escape\"",
		"\"is\"",
		"\"empty\"",
		"\"null\"",
		"\"member\"",
		"\"of\"",
		"DOT",
		"\"concat\"",
		"\"substring\"",
		"\"length\"",
		"\"locate\"",
		"\"abs\"",
		"\"sqrt\"",
		"\"mod\"",
		"an integer literal",
		"FLOAT_LITERAL",
		"\"true\"",
		"\"false\"",
		"EXPLICIT_BRACKETING",
		"UNARY_PLUS",
		"UNARY_MINUS",
		"\"unknown\"",
		"WS",
		"QUESTION",
		"DIGIT",
		"OCTAL_DIGIT",
		"HEX_DIGIT",
		"INTEGER_SUFFIX",
		"FLOAT_SUFFIX",
		"EXPONENT"
	};
	
	private static final long[] mk_tokenSet_0() {
		long[] data = { -1152877524141735936L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_0 = new BitSet(mk_tokenSet_0());
	}
	
