/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejbqlcompiler;

import javax.sql.DataSource;

import org.tranql.ql.Function;
import org.tranql.ql.Join;
import org.tranql.ql.Node;
import org.tranql.ql.QueryException;
import org.tranql.sql.sql92.SQL92Generator;

/**
 * @version $Revision: 183 $ $Date: 2005-03-11 20:25:48 -0500 (Fri, 11 Mar 2005) $
 */
public class DerbyGenerator extends SQL92Generator {

    public DerbyGenerator(DataSource ds) {
        super(ds);
    }

    public Object visit(Function node, Object param) throws QueryException {
        Context ctx = (Context) param;
        
        if (Function.CONCAT == node.getFunctionType() || Function.SUBSTRING == node.getFunctionType()) {
            ctx.append("{fn ");
            ctx.append(node.getFunctionType().getName());
            generateList(node, param);
            ctx.append("}");
        } else {
            ctx.append(node.getFunctionType().getName());
            generateList(node, param);
        }
        return param;
    }
    
    public Object visit(Join node, Object param) throws QueryException {
        Context ctx = (Context) param;
        Node left = node.getChild();
        left.visit(this, param);
        if (node.getType() == Join.CROSS_JOIN) {
            ctx.append(", " );
        } else {
            ctx.append(JOIN_TYPES.get(node.getType()));
        }
        Node right = left.getSibling();
        right.visit(this, param);
        Node condition = right.getSibling();
        if (condition != null) {
            ctx.append(" ON (");
            condition.visit(this, param);
            ctx.append(')');
        }
        return param;
    }
    
}
