/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejbqlcompiler;

import org.tranql.ql.Node;
import org.tranql.ql.OrderBy;
import org.tranql.ql.Query;
import org.tranql.ql.QueryException;
import org.tranql.ql.Where;

/**
 * Append an empty WHERE clause, if such a clause does not already exist.
 * 
 * @version $Revision: 183 $ $Date: 2005-03-11 20:25:48 -0500 (Fri, 11 Mar 2005) $
 */
class EmptyWhereAppender extends InterRepresentationTransformer {
    private boolean isWhereDefined = false;
    
    public EmptyWhereAppender() {
        super(null);
    }

    public Object visit(Query node, Object param) throws QueryException {
        Node query = (Node) super.visit(node, param);
        if (isWhereDefined) {
            return query;
        }
        query.addChild(new Where());
        return query;
    }

    public Object visit(Where node, Object param) throws QueryException {
        isWhereDefined = true;
        return super.visit(node, param);
    }
    
    public Object visit(OrderBy node, Object param) throws QueryException {
        if (isWhereDefined) {
            return super.visit(node, param);
        }
        
        isWhereDefined = true;
        Where where = new Where();
        Node orderBy = (Node) super.visit(node, param);
        where.setSibling(orderBy);
        
        return where;
    }
}
