/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejbqlcompiler;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.tranql.builder.QueryTileBuilder;
import org.tranql.ql.AliasedEntity;
import org.tranql.ql.AttributeReference;
import org.tranql.ql.BinaryOperation;
import org.tranql.ql.DerivedTable;
import org.tranql.ql.FKAttributeReference;
import org.tranql.ql.Join;
import org.tranql.ql.Node;
import org.tranql.ql.Path;
import org.tranql.ql.QueryException;
import org.tranql.ql.QuerySource;
import org.tranql.ql.Where;
import org.tranql.schema.Association;
import org.tranql.schema.AssociationEnd;
import org.tranql.schema.Attribute;
import org.tranql.schema.Entity;
import org.tranql.schema.FKAttribute;

/**
 * This visitor injects the registered DerivedTables.
 * 
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
class InjectRegisteredDerivedTablesInnerJoinVisitor extends InterRepresentationTransformer {
    private final InjectRegisteredDerivedTablesVisitorContext ctx;
    
    public InjectRegisteredDerivedTablesInnerJoinVisitor(InjectRegisteredDerivedTablesVisitorContext ctx) {
        super(null);
        this.ctx = ctx;
    }
    
    public Object visit(DerivedTable node, Object param) throws QueryException {
        Path path = node.getPath();
        List thePath = path.getPath();

        int index = 0;
        QuerySource previousQuerySource = ctx.factory(index, node);
        Node toNode = null;
        for (int i = 0; i < thePath.size(); i++) {
            AssociationEnd end = (AssociationEnd) thePath.get(i);
            index++;
            QuerySource querySource = ctx.factory(index, node);

            if (null == toNode) {
                toNode = new Join(Join.INNER_JOIN).addChild(previousQuerySource).addChild(querySource);
            } else {
                toNode = new Join(Join.INNER_JOIN).addChild(toNode).addChild(querySource);
            }
            
            Node condition = null;
            if (end.isManyToMany()) {
                throw new UnsupportedOperationException("MTM are not yet supported");
            }
            Association.JoinDefinition joinDefinition = end.getAssociation().getJoinDefinition();
            QuerySource pkQuerySource;
            QuerySource fkQuerySource;
            if ( previousQuerySource.getEntity() == joinDefinition.getPKEntity()) {
                pkQuerySource = previousQuerySource;
                fkQuerySource = querySource;
            } else {
                pkQuerySource = querySource;
                fkQuerySource = previousQuerySource;
            }

            LinkedHashMap pkToFkMapping = joinDefinition.getPKToFKMapping();
            for (Iterator iterator = pkToFkMapping.entrySet().iterator(); iterator.hasNext();) {
                Map.Entry element = (Map.Entry) iterator.next();
                Attribute attribute = (Attribute) element.getKey();
                FKAttribute fkAttribute = (FKAttribute) element.getValue();
                Node localCondition = new BinaryOperation(BinaryOperation.EQ);
                localCondition.addChild(new AttributeReference(pkQuerySource, attribute));
                localCondition.addChild(new FKAttributeReference(fkQuerySource, fkAttribute));
                condition = null == condition ? localCondition : new BinaryOperation(BinaryOperation.AND).addChild(condition).addChild(localCondition);
            }
            toNode.addChild(condition);
            
            previousQuerySource = querySource;
        }
        
        return toNode;
    }
    
    public Object visit(Where node, Object param) throws QueryException {
        Collection derivedTables = ctx.getRegisteredDerivedTables();
        
        BinaryOperation operation = null;
        for (Iterator iter = derivedTables.iterator(); iter.hasNext();) {
            DerivedTable derivedTable = (DerivedTable) iter.next();
            AliasedEntity sourceEntity = derivedTable.getPath().getRoot();
            Entity entity = sourceEntity.getEntity();
            AliasedEntity clonedEntity = ctx.factory(0, derivedTable);
            List attributes = entity.getAttributes();
            for (Iterator iterator = attributes.iterator(); iterator.hasNext();) {
                Attribute attribute = (Attribute) iterator.next();
                if (attribute.isIdentity()) {
                    BinaryOperation joinPKToFK = new BinaryOperation(BinaryOperation.EQ);
                    joinPKToFK.addChild(new AttributeReference(sourceEntity, attribute));
                    joinPKToFK.addChild(new AttributeReference(clonedEntity, attribute));
                    operation = null == operation ? joinPKToFK : (BinaryOperation) new BinaryOperation(BinaryOperation.AND).addChild(joinPKToFK).addChild(operation);
                }
            }
        }

        return QueryTileBuilder.buildWhere(node, operation, this, param);
    }

}
