/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejbqlcompiler;

import java.util.List;

import org.tranql.ejbqlcompiler.StackNodeRegister.NodeRegister;
import org.tranql.ql.AggregateFunction;
import org.tranql.ql.CountAggregateFunction;
import org.tranql.ql.Empty;
import org.tranql.ql.IsNull;
import org.tranql.ql.MemberOf;
import org.tranql.ql.Path;
import org.tranql.ql.QueryException;
import org.tranql.ql.Select;
import org.tranql.schema.AssociationEnd;

/**
 * 
 * @version $Revision: 183 $ $Date: 2005-03-12 12:25:48 +1100 (Sat, 12 Mar 2005) $
 */
public class PathSearcher extends AbstractNodeSearcher {
    public static final PathSearcher PATH_SEARCHER = new PathSearcher();
    
    private static final Object COUNT_DISTINCT_MULTI_PK_FIELDS = new Object();
    private static final Object SELECTED_PATH = new Object();
    private static final Object PATH = new Object();

    public boolean isFoundCountDistinctMultiPKFields(NodeRegister register) {
        Boolean countMultiPkFields = (Boolean) register.getSingle(PathSearcher.COUNT_DISTINCT_MULTI_PK_FIELDS);
        if (null == countMultiPkFields) {
            countMultiPkFields = Boolean.FALSE;
        }
        return countMultiPkFields.booleanValue();
    }

    public Path getFoundSelectedPath(NodeRegister register) {
        return (Path) register.getSingle(PathSearcher.SELECTED_PATH);
    }

    public List getFoundPaths(NodeRegister register) {
        return (List) register.get(PathSearcher.PATH);
    }

    public Object visit(Select node, Object param) throws QueryException {
        if (!(node.getChild() instanceof Path)) {
            return super.visit(node, param);
        }

        StackNodeRegister register = (StackNodeRegister) param;
        register.set(SELECTED_PATH, node.getChild());
        return param;
    }
    
    public Object visit(AggregateFunction node, Object param) throws QueryException {
        StackNodeRegister register = (StackNodeRegister) param;
        register.set(SELECTED_PATH, node.getPath());
        return param;
    }
    
    public Object visit(CountAggregateFunction node, Object param) throws QueryException {
        if (node.isEntityReferenceCounted()) {
            return param;
        }
        
        Path path = node.getPath();
        StackNodeRegister register = (StackNodeRegister) param;
        if (node.isDistinct()) {
            if (path.isEndAssociationEnd()) {
                AssociationEnd end = path.getEndAssociationEnd();
                if (1 < end.getEntity().getPrimaryKeyFields().size()) {
                    register.set(COUNT_DISTINCT_MULTI_PK_FIELDS, Boolean.TRUE);
                }
            }
        }
        register.set(SELECTED_PATH, node.getPath());
        return param;
    }
    
    public Object visit(Path node, Object param) throws QueryException {
        StackNodeRegister register = (StackNodeRegister) param;
        register.put(PATH, node);
        return param;
    }

    public Object visit(IsNull node, Object param) throws QueryException {
        // Implementation Note: intentionally skip Paths in IsNull. They have a specific processing.
        return param;
    }
    
    public Object visit(MemberOf node, Object param) throws QueryException {
        // Implementation Note: intentionally skip Paths in MemberOf. They have a specific processing.
        return param;
    }
    
    public Object visit(Empty node, Object param) throws QueryException {
        // Implementation Note: intentionally skip Paths in Empty. They have a specific processing.
        return param;
    }

}