/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ejbqlcompiler;

import java.util.ArrayList;
import java.util.Collections;
import java.util.IdentityHashMap;
import java.util.List;
import java.util.Stack;

import org.tranql.ql.Node;


/**
 * 
 * @version $Revision: 183 $ $Date: 2005-03-11 20:25:48 -0500 (Fri, 11 Mar 2005) $
 */
class StackNodeRegister {
    private final IdentityHashMap nodeToRegisters = new IdentityHashMap();
    private final Stack stackNodes = new Stack();
    private final List nodes = new ArrayList();
    
    public void push(Object node) {
        nodes.add(node);
        stackNodes.push(node);
        nodeToRegisters.put(node, new NodeRegisterImpl());
    }
    
    public Node pop() {
        return (Node) stackNodes.pop();
    }
    
    public Node peek() {
        return (Node) stackNodes.peek();
    }

    public void set(Object key, Object node) {
        NodeRegisterImpl register = (NodeRegisterImpl) nodeToRegisters.get(stackNodes.peek());
        register.set(key, node);
    }

    public void put(Object key, Object node) {
        NodeRegisterImpl register = (NodeRegisterImpl) nodeToRegisters.get(stackNodes.peek());
        register.put(key, node);
    }

    public IdentityHashMap getRegisters() {
        return nodeToRegisters;
    }
    
    public interface NodeRegister {
        public List get(Object key);
        
        public Object getSingle(Object key);
    }
    
    private static final class NodeRegisterImpl implements NodeRegister {
        private final IdentityHashMap keyToNodes = new IdentityHashMap();
        
        private void put(Object key, Object node) {
            List nodes = (List) keyToNodes.get(key);
            if (null == nodes) {
                nodes = new ArrayList();
                keyToNodes.put(key, nodes);
            }
            nodes.add(node);
        }

        public List get(Object key) {
            List returned = (List) keyToNodes.get(key);
            if (null != returned) {
                return Collections.unmodifiableList(returned);
            }
            return Collections.EMPTY_LIST;
        }
        
        private void set(Object key, Object node) {
            keyToNodes.put(key, node);
        }
        
        public Object getSingle(Object key) {
            return keyToNodes.get(key);
        }
    }
    
}
