/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.field;

import java.io.Serializable;


/**
 * Transform that converts values being loaded or stored into the cache.
 *
 * @version $Revision: 45 $ $Date: 2004-04-19 18:23:20 -0400 (Mon, 19 Apr 2004) $
 */
public interface FieldTransform extends Serializable {
    /**
     * Return the Java Class that will be returned by fetch and that should be passed to set.
     * @return the Java Class for this field
     */
    Class getFieldClass();

    /**
     * Return a value from the cache
     * @param row the row to fetch from
     * @return the transformed value
     */
    Object get(Row row) throws FieldTransformException;

    /**
     * Set a value into the cache
     * @param row the row to store the value into
     * @param value the value to be transformed
     */
    void set(Row row, Object value) throws FieldTransformException;
}
