/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.identity;

import java.util.Arrays;

/**
 * @version $Revision: 104 $ $Date: 2004-09-09 01:24:10 -0400 (Thu, 09 Sep 2004) $
 */
public final class DerivedIdentityValue {
    private final Object[] values;
    private final int hashCode;

    public DerivedIdentityValue(Object[] values) {
        this.values = values;
        int hc = 17;
        for (int i = 0; i < values.length; i++) {
            Object value = values[i];
            hc = hc * 37 + value.hashCode();
        }
        hashCode = hc;
    }

    public Object[] getValues() {
        return values;
    }

    public int hashCode() {
        return hashCode;
    }

    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        } else if (obj instanceof DerivedIdentityValue) {
            return Arrays.equals(this.values, ((DerivedIdentityValue)obj).values);
        } else {
            return false;
        }
    }
}
