/**
 * 
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.intertxcache;

import org.tranql.cache.CacheRow;
import org.tranql.cache.InTxCache;
import org.tranql.ejb.CMPFieldTransform;

/**
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class CacheFieldFaultTransform implements CMPFieldTransform {
    private final FrontEndCache cache;
    private final CMPFieldTransform next;
    private final int indices[];
    
    public CacheFieldFaultTransform(FrontEndCache cache, CMPFieldTransform next, int indice) {
        this.cache = cache;
        this.next = next;
        this.indices = new int[] {indice};
    }

    public Object get(InTxCache inTxCache, CacheRow row) {
        try {
            boolean allGet = cache.get(row, indices);
            if (allGet) {
                return row.get(indices[0]);
            }
            Object opaque = next.get(inTxCache, row);
            cache.set(row, indices);
            return opaque;
        } catch (CacheException e) {
            throw new RuntimeCacheException(e);
        }
    }

    public void set(InTxCache inTxCache, CacheRow row, Object value) {
        next.set(inTxCache, row, value);
        try {
            cache.set(row, indices);
        } catch (CacheException e) {
            throw new RuntimeCacheException(e);
        }
    }
}