/**
 * 
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.intertxcache;

import org.tranql.cache.CacheRow;
import org.tranql.cache.InTxCache;
import org.tranql.field.Row;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityTransform;
import org.tranql.identity.IdentityTransformException;
import org.tranql.ql.Query;
import org.tranql.ql.QueryException;
import org.tranql.query.QueryCommand;
import org.tranql.query.ResultHandler;


/**
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class FindByPKCacheQueryCommand implements QueryCommand {
    private final FrontEndCache cache;
    private final IdentityTransform identityTransform;
    private final QueryCommand next;
    
    public FindByPKCacheQueryCommand(FrontEndCache cache, IdentityTransform identityTransform, QueryCommand next) {
        this.cache = cache;
        this.identityTransform = identityTransform;
        this.next = next;
    }
    
    public Query getQuery() {
        return next.getQuery();
    }

    public Object execute(InTxCache cache, ResultHandler handler, Row params, Object arg) throws QueryException {
        Object pk = params.get(0);
        try {
            GlobalIdentity id = identityTransform.getGlobalIdentity(pk);
            CacheRow cacheRow = cache.get(id);
            if (null != cacheRow) {
                Row row = new Row(new Object[] {pk});
                arg = handler.fetched(row, arg);
                arg= handler.endFetched(arg);
                return arg;
            }
        } catch (IdentityTransformException e) {
            throw new QueryException(e);
        }
        
        return next.execute(cache, handler, params, arg);
    }
}