/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import org.tranql.schema.Attribute;

/**
 * This node represents an attribute navigation of an entity.
 * This node expects no children.
 *
 * someEntity.someAttribute
 *
 * @version $Revision: 138 $ $Date: 2004-10-20 23:52:45 -0400 (Wed, 20 Oct 2004) $
 */
public class AttributeReference extends AbstractNode {
    private final AliasedEntity source;
    private final Attribute attribute;

    public AttributeReference(AliasedEntity source, Attribute attribute) {
        this.source = source;
        this.attribute = attribute;
    }

    public AliasedEntity getSource() {
        return source;
    }

    public Attribute getAttribute() {
        return attribute;
    }

    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
}
