/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

/**
 * This node is used in a where clause to represent a binary logical
 * comparison, arithmetic operation or a equivlency comparison. This
 * node expects two children a left value, expression or literal and
 * a right value, expression or literal.
 *
 * left AND right
 * left + right
 * left <= right
 *
 * @version $Revision: 138 $ $Date: 2004-10-20 23:52:45 -0400 (Wed, 20 Oct 2004) $
 */
public class BinaryOperation extends AbstractNode {
    public static final String EQ = "=";
    public static final String NE = "<>";
    public static final String LT = "<";
    public static final String LE = "<=";
    public static final String GT = ">";
    public static final String GE = ">=";
    public static final String AND = "AND";
    public static final String OR = "OR";
    public static final String PLUS = "+";
    public static final String MINUS = "-";
    public static final String TIMES = "*";
    public static final String DIV = "/";
    public static final String IN = "IN";

    private final String operation;

    public BinaryOperation(String operation) {
        this.operation = operation;
    }

    public String getOperation() {
        return operation;
    }

    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
}
