/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import java.util.List;

import org.tranql.schema.AssociationEnd;
import org.tranql.schema.Entity;


/**
 * This class is really intended to be used for SQL, but
 * for now we are using it for the Linda IN clause in EJB-QL.
 *
 * @version $Revision: 233 $ $Date: 2005-08-19 23:48:40 -0400 (Fri, 19 Aug 2005) $
 */
public final class DerivedTable extends AliasedEntity {
    private final Path path;
    private final String alias;

    public DerivedTable(Path path, String alias) {
        super(getEntityType(path), alias);
        this.path = path;
        this.alias = alias;
    }

    public Path getPath() {
        return path;
    }

    public String getAlias() {
        return alias;
    }

    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }

    private static Entity getEntityType(Path path) {
        List pathList = path.getPath();
        Object lastElement = pathList.get(pathList.size()-1);
        if (!(lastElement instanceof AssociationEnd)) {
            throw new MalformedNodeException("Path used must end with a CMR field");
        }
        AssociationEnd end = (AssociationEnd) lastElement;
        if (end.isManyToOne() || end.isOneToOne()) {
            throw new MalformedNodeException("Path used must end with a multi valued CMR field");
        }
        return end.getEntity();
    }
}
