/**
 *
 *  Copyright 2004 Dain Sundstrom
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import java.util.List;

import org.tranql.schema.AssociationEnd;

/**
 * @version $Revision: 233 $ $Date: 2005-08-19 23:48:40 -0400 (Fri, 19 Aug 2005) $
 */
public class Empty extends AbstractNode {
    private final boolean not;

    public Empty(boolean not) {
        this.not = not;
    }

    public boolean isNot() {
        return not;
    }

    public Path getPath() {
        return (Path) getChild();
    }
    
    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
    
    public Node addChild(Node child) {
        if (false == child instanceof Path) {
            throw new MalformedNodeException("Child is not a Path expression");
        }
        Path path = (Path) child;
        List pathList = path.getPath();
        Object lastElement = pathList.get(pathList.size()-1);
        if (!(lastElement instanceof AssociationEnd)) {
            throw new MalformedNodeException("Path used must end with a CMR field");
        }
        AssociationEnd end = (AssociationEnd) lastElement;
        if (end.isManyToOne() || end.isOneToOne()) {
            throw new MalformedNodeException("Path used must end with a multi valued CMR field");
        }
        return super.addChild(child);
    }
    
}
