/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import java.io.Serializable;

/**
 * This node represents a function call.  This is normally used within a
 * where clause.  This node expects zero or more children.
 *
 * abs(-42)
 *
 * @version $Revision: 173 $ $Date: 2005-02-20 10:42:56 -0500 (Sun, 20 Feb 2005) $
 */
public class Function extends AbstractNode {
    public static final FunctionType CONCAT = new FunctionType("CONCAT");
    public static final FunctionType SUBSTRING = new FunctionType("SUBSTRING");
    public static final FunctionType LENGTH = new FunctionType("LENGTH");
    public static final FunctionType LOCATE = new FunctionType("LOCATE");
    public static final FunctionType ABS = new FunctionType("ABS");
    public static final FunctionType SQRT = new FunctionType("SQRT");
    public static final FunctionType MOD = new FunctionType("MOD");
    
    public static class FunctionType implements Serializable {
        private String name;
        private FunctionType(String name) {
            this.name = name;
        }
        public String getName() {
            return name;
        }
    }

    private final FunctionType functionType;

    public Function(FunctionType functionType) {
        this.functionType = functionType;
    }

    public FunctionType getFunctionType() {
        return functionType;
    }

    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
}
