/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import java.io.Serializable;



/**
 * This node represents a JOIN cluase in the FROM clause of a
 * SELECT statement.  This node supports cross, inner, left
 * right and full join.  This node expects two or three childred
 * a left entity, right entity, and an optional condition.
 *
 *  left INNER JOIN right ON (left.x = right.y) 
 *
 * @version $Revision: 176 $ $Date: 2005-02-27 02:20:24 -0500 (Sun, 27 Feb 2005) $
 */
public final class Join extends AbstractNode {
    public static final JoinType CROSS_JOIN = new JoinType();
    public static final JoinType INNER_JOIN = new JoinType();
    public static final JoinType LEFT_JOIN = new JoinType();
    public static final JoinType RIGHT_JOIN = new JoinType();
    public static final JoinType FULL_JOIN = new JoinType();

    private static class JoinType implements Serializable {
    }

    private final JoinType type;

    public Join(JoinType type) {
        this.type = type;
    }

    public JoinType getType() {
        return type;
    }

    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
}
