/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import java.util.Iterator;
import java.util.List;

import org.tranql.schema.AssociationEnd;
import org.tranql.schema.Attribute;
import org.tranql.schema.Entity;



/**
 * This node represents a navigation in the object graph.
 *
 * a.b.c.d
 *
 * @version $Revision: 410 $ $Date: 2006-10-21 20:15:17 -0400 (Sat, 21 Oct 2006) $
 */
public class Path extends AbstractNode {
    private final AliasedEntity root;
    private final List path;

    public Path(AliasedEntity root, List path) {
        if (null == root) {
            throw new IllegalArgumentException("root is required");
        } else if (null == path || path.isEmpty()) {
            throw new IllegalArgumentException("path is required");
        }
        this.root = root;
        this.path = path;
        checkSemantic();
    }

    public AliasedEntity getRoot() {
        return root;
    }

    public List getPath() {
        return path;
    }

    public boolean isEndAttribute() {
        return path.get(path.size() - 1) instanceof Attribute;
    }
    
    public boolean isEndAssociationEnd() {
        return !isEndAttribute();
    }

    public Object getEnd() {
        return path.get(path.size() - 1);
    }

    public Attribute getEndAttribute() {
        return (Attribute) getEnd();
    }

    public AssociationEnd getEndAssociationEnd() {
        return (AssociationEnd) getEnd();
    }

    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
    
    private void checkSemantic() {
        boolean isMultiValuedCMR = false;
        Entity currentEntity = root.getEntity();
        for (Iterator iter = path.iterator(); iter.hasNext();) {
            if (currentEntity == null) {
                throw new MalformedNodeException("A cmp-field path can not be further navigated. Path is " + path);
            } else if (isMultiValuedCMR) {
                throw new MalformedNodeException("A multi valued cmr-field path can not be further navigated. Path is " + path);
            }
            Object o = iter.next();
            if (o instanceof AssociationEnd) {
                AssociationEnd end = (AssociationEnd) o;
                isMultiValuedCMR = end.isOneToMany() || end.isManyToMany();
                currentEntity = end.getEntity();
            } else if (o instanceof Attribute) {
                currentEntity = null;
            }
        }
    }
}
