/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import org.tranql.field.FieldTransform;


/**
 * This node represents a select query.
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class Query extends AbstractNode {
    private final QueryBinding[] paramTransforms;
    private final QueryBinding[] resultTransforms;
    private FieldTransform[] resultAccessors;
    
    public Query(QueryBinding[] paramTransforms, QueryBinding[] resultTransforms) {
        this.paramTransforms = paramTransforms;
        this.resultTransforms = resultTransforms;
    }
    
    public Query(Query prototype) {
        this.paramTransforms = prototype.paramTransforms;
        this.resultTransforms = prototype.resultTransforms;
        this.resultAccessors = prototype.resultAccessors;
    }

    public QueryBinding[] getParamTransforms() {
        return paramTransforms;
    }

    public QueryBinding getParamTransform(int slot) throws QueryException {
        if (paramTransforms.length < slot + 1) {
            throw new QueryException("Only " + paramTransforms.length + " parameters are defined.");
        }
        return paramTransforms[slot];
    }
    
    public QueryBinding[] getResultTransforms() {
        return resultTransforms;
    }

    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
    
    public void setResultAccessors(FieldTransform[] resultAccessors) {
        this.resultAccessors = resultAccessors;
    }
    
    public FieldTransform[] getResultAccessors() {
        return resultAccessors;
    }
}
