/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import org.tranql.field.FieldAccessor;
import org.tranql.field.FieldTransform;
import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;
import org.tranql.schema.Attribute;
import org.tranql.schema.Entity;
import org.tranql.schema.FKAttribute;


/**
 *
 *
 * @version $Revision: 165 $ $Date: 2005-02-01 05:44:57 -0500 (Tue, 01 Feb 2005) $
 */
public class QueryBindingImpl implements QueryBinding {
    private final FieldTransform next;
    private Entity entity;
    private Attribute attribute;
    private FKAttribute fkAttribute;

    public QueryBindingImpl(FieldTransform next, Entity entity, Attribute attribute) {
        this.next = next;
        this.entity = entity;
        this.attribute = attribute; 
    }

    public QueryBindingImpl(FieldTransform next, Entity entity, FKAttribute fkAttribute) {
        this.next = next;
        this.entity = entity;
        this.fkAttribute = fkAttribute; 
    }

    public QueryBindingImpl(FieldTransform next) {
        this.next = next;
    }

    public QueryBindingImpl(int slot, Entity entity, Attribute attribute) {
        next = new FieldAccessor(slot, attribute.getType());
        this.entity = entity;
        this.attribute = attribute; 
        fkAttribute = null;
    }

    public QueryBindingImpl(int slot, Entity entity, FKAttribute fkAttribute) {
        next = new FieldAccessor(slot, fkAttribute.getType());
        this.entity = entity;
        this.fkAttribute = fkAttribute;
        attribute = null;
    }

    public QueryBindingImpl(int slot, Class fieldClass) {
        next = new FieldAccessor(slot, fieldClass);
        attribute = null;
        fkAttribute = null;
    }

    public void setEntity(Entity entity) {
        this.entity = entity;
    }
    
    public Entity getEntity() {
        return entity;
    }
    
    public boolean isAttributeBinding() {
        return null != attribute;
    }

    public void setAttribute(Attribute attribute) {
        if (null != fkAttribute) {
            throw new IllegalStateException();
        }
        this.attribute = attribute;
    }
    
    public Attribute getAttribute() {
        return attribute;
    }
    
    public boolean isFKAttributeBinding() {
        return null != fkAttribute;
    }
    
    public void setFKAttribute(FKAttribute fkAttribute) {
        if (null != attribute) {
            throw new IllegalStateException();
        }
        this.fkAttribute = fkAttribute;
    }
    
    public FKAttribute getFKAttribute() {
        return fkAttribute;
    }
    
    public boolean isUnknownBinding() {
        return null == attribute && null == fkAttribute;
    }

    public Class getFieldClass() {
        if (null != attribute) {
            return attribute.getType();
        } else if (null != fkAttribute) {
            return fkAttribute.getType();
        }
        return next.getFieldClass();
    }

    public Object get(Row row) throws FieldTransformException {
        return next.get(row);
    }

    public void set(Row row, Object value) throws FieldTransformException {
        next.set(row, value);
    }
}
