/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.ql;

import org.tranql.schema.Entity;

/**
 * This subclass is used for select queries and adds a forUpdate flag.
 * If the for update flag is set, an exclusive lock will be acquired
 * on the entities selected.
 *
 * @version $Revision: 133 $ $Date: 2004-10-19 16:42:06 -0400 (Tue, 19 Oct 2004) $
 */
public final class QuerySource extends AliasedEntity {
    private final boolean forUpdate;

    public QuerySource(Entity entity, String alias) {
        super(entity, alias);
        this.forUpdate = false;
    }

    public QuerySource(Entity entity, String alias, boolean forUpdate) {
        super(entity, alias);
        this.forUpdate = forUpdate;
    }

    public boolean isForUpdate() {
        return forUpdate;
    }

    public Object visit(QueryVisitor visitor, Object param) throws QueryException {
        return visitor.visit(this, param);
    }
}
