/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql;

import org.tranql.schema.FKAttribute;
import org.tranql.sql.TypeConverter;

/**
 *
 *
 * @version $Revision: 323 $ $Date: 2006-02-19 01:29:24 -0500 (Sun, 19 Feb 2006) $
 */
public class FKColumn implements FKAttribute {
    private final String name;
    private final String physicalName;
    private final Class type;
    private boolean isSQLTypeSet;
    private int sqlType;
    private int precision = -1;
    private int scale = -1;
    private int length = -1;
    private String columnDefinition;
    private TypeConverter typeConverter;

    public FKColumn(String name, String physicalName, Class type) {
        this.name = name;
        this.physicalName = physicalName;
        this.type = type;
    }

    public String getName() {
        return name;
    }

    public String getPhysicalName() {
        return physicalName;
    }

    public Class getType() {
        return type;
    }

    public boolean isSQLTypeSet() {
        return isSQLTypeSet;
    }
    
    public void setSQLType(int sqlType) {
        if (isSQLTypeSet) {
            throw new IllegalStateException("Can't overwrite SQL type");
        }
        isSQLTypeSet = true;
        this.sqlType = sqlType;
    }

    public int getSQLType() {
        return sqlType;
    }
    
    public int getLength() {
        return length;
    }

    public void setLength(int length) {
        this.length = length;
    }

    public int getPrecision() {
        return precision;
    }

    public void setPrecision(int precision) {
        this.precision = precision;
    }

    public int getScale() {
        return scale;
    }

    public void setScale(int scale) {
        this.scale = scale;
    }

    public String getColumnDefinition() {
        return columnDefinition;
    }

    public boolean isColumnDefinitionSet() {
        return null != columnDefinition;
    }

    public void setColumnDefinition(String columnDefinition) {
        this.columnDefinition = columnDefinition;
    }

    public boolean isTypeConverterSet() {
        return null != typeConverter;
    }
    
    public void setTypeConverter(TypeConverter typeConverter) {
        if (null != this.typeConverter) {
            throw new IllegalStateException("Can't overwrite type converter");
        }
        this.typeConverter = typeConverter;
    }

    public TypeConverter getTypeConverter() {
        return typeConverter;
    }
}
