/**
 *
 *  Copyright 2005 Matt Hogstrom
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.db2;

import javax.sql.DataSource;

import org.tranql.ql.ConditionalAssignment;
import org.tranql.ql.Join;
import org.tranql.ql.Node;
import org.tranql.ql.Query;
import org.tranql.ql.QueryException;
import org.tranql.sql.sql92.SQL92Generator;
import org.tranql.sql.sql92.SQL92Generator.Context;


/**
 * @version $Revision: 433 $ $Date: 2006-12-12 00:18:26 -0500 (Tue, 12 Dec 2006) $
 */
public class DB2Generator extends SQL92Generator {

    public DB2Generator(DataSource ds) {
        super(ds);
    }

    public Object visit(ConditionalAssignment node, Object param) throws QueryException {
        Context ctx = (Context) param;
        Node set = node.getChild();
        Node flag = set.getSibling();
        Node value = flag.getSibling();
        set.visit(this, param);
        ctx.append(" = CASE WHEN 1 = ");
        flag.visit(this, param);
        ctx.append(" THEN ");
        value.visit(this, param);
        ctx.append(" ELSE ");
        set.visit(this, param);
        ctx.append(" END");
        return param;
    }
    
    // Added CROSS JOIN as a comma as DB2 does not support CROSS JOIN SYNTAX    
    public Object visit(Join node, Object param) throws QueryException {
        Context ctx = (Context) param;
        Node left = node.getChild();
        left.visit(this, param);
        if (node.getType() == Join.CROSS_JOIN) {
            ctx.append(", " );
        } else {
            ctx.append(JOIN_TYPES.get(node.getType()));
        }
        Node right = left.getSibling();
        right.visit(this, param);
        Node condition = right.getSibling();
        if (condition != null) {
            ctx.append(" ON (");
            condition.visit(this, param);
            ctx.append(')');
        }
        return param;
    }
    

    public Object visit(Query node, Object param) throws QueryException {
        Context ctx = (Context) param;
        for (Node child = node.getChild(); child != null; child = child.getSibling()) {
            param = child.visit(this, param);
        }
        if (ctx.isForUpdate()) {
            ctx.append(" FOR UPDATE WITH RS USE AND KEEP UPDATE LOCKS");
        }
        return param;
    }
    
}
