/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.jdbc.binding;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;
import org.tranql.ql.QueryBinding;
import org.tranql.schema.Attribute;
import org.tranql.schema.Entity;
import org.tranql.schema.FKAttribute;
import org.tranql.sql.jdbc.InputBinding;
import org.tranql.sql.jdbc.OutputBinding;
import org.tranql.sql.jdbc.ResultBinding;

/**
 *
 *
 * @version $Revision: 155 $ $Date: 2004-12-02 18:12:10 -0500 (Thu, 02 Dec 2004) $
 */
public abstract class AbstractBinding implements ResultBinding, InputBinding, OutputBinding {
    protected final int index;
    protected final QueryBinding transform;

    public AbstractBinding(int index, QueryBinding transform) {
        this.index = index;
        this.transform = transform;
    }

    public void getValue(ResultSet rs, Row row) throws SQLException, FieldTransformException {
        transform.set(row, getValue(rs));
    }

    public void setValue(PreparedStatement ps, Row row) throws SQLException, FieldTransformException {
        setValue(ps, transform.get(row));
    }
    
    public Attribute getAttribute() {
        return transform.getAttribute();
    }

    public Entity getEntity() {
        return transform.getEntity();
    }

    public FKAttribute getFKAttribute() {
        return transform.getFKAttribute();
    }

    public boolean isAttributeBinding() {
        return transform.isAttributeBinding();
    }

    public boolean isFKAttributeBinding() {
        return transform.isFKAttributeBinding();
    }

    public boolean isUnknownBinding() {
        return transform.isUnknownBinding();
    }

    public void setAttribute(Attribute attribute) {
        throw new UnsupportedOperationException();
    }

    public void setEntity(Entity entity) {
        throw new UnsupportedOperationException();
    }

    public void setFKAttribute(FKAttribute fkAttribute) {
        throw new UnsupportedOperationException();
    }
}
