/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.jdbc.binding;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ByteArrayInputStream;

import org.tranql.ql.QueryBinding;

/**
 *
 *
 *
 * @version $Revision: 221 $ $Date: 2005-07-31 15:30:00 -0400 (Sun, 31 Jul 2005) $
 */
public class ByteArrayBinding extends AbstractBinding {
    public ByteArrayBinding(int index, QueryBinding transform) {
        super(index, transform);
    }

    public Object getValue(ResultSet rs) throws SQLException {
        InputStream is = rs.getBinaryStream(index);
        if (rs.wasNull()) {
            return null;
        }
        return readStream(is);
    }

    public void setValue(PreparedStatement ps, Object value) throws SQLException {
        if (value == null) {
            ps.setNull(index, getSQLType());
        } else {
            byte[] bytes = (byte[]) value;
            ps.setBinaryStream(index, new ByteArrayInputStream(bytes), bytes.length);
        }
    }

    public void register(CallableStatement cs) throws SQLException {
        cs.registerOutParameter(index, getSQLType());
    }

    public Object getValue(CallableStatement cs) throws SQLException {
        // can't read a binary stream from a CallableStatement
        throw new UnsupportedOperationException();
    }
    
    public int getSQLType() {
        return Types.BLOB;
    }

    private Object readStream(InputStream is) throws SQLException {
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        byte[] buffer = new byte[4096];
        int count = 0;
        try {
            while ((count = is.read(buffer)) > 0) {
                os.write(buffer, 0, count);
            }
        } catch (IOException e) {
            throw (SQLException) new SQLException(e.getMessage()).initCause(e);
        }
        return os.toByteArray();
    }
}
