/**
 *
 *  Copyright 2005 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.jdbc.binding;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.tranql.ql.QueryBinding;

/**
 *
 *
 *
 * @version $Revision: 282 $ $Date: 2005-10-18 20:51:37 -0400 (Tue, 18 Oct 2005) $
 */
public class CharBinding extends AbstractBinding {
    public CharBinding(int index, QueryBinding transform) {
        super(index, transform);
    }

    public Object getValue(ResultSet rs) throws SQLException {
        String value = rs.getString(index);
        return convertToCharacter(value);
    }

    public void setValue(PreparedStatement ps, Object value) throws SQLException {
        Character character = (Character) value;
        if (null == character) {
            ps.setString(index, null);
        } else {
            ps.setString(index, character.toString());
        }
    }

    public void register(CallableStatement cs) throws SQLException {
        cs.registerOutParameter(index, getSQLType());
    }

    public Object getValue(CallableStatement cs) throws SQLException {
        String value = cs.getString(index);
        return convertToCharacter(value);
    }
    
    public int getSQLType() {
        return Types.CHAR;
    }
    
    private Character convertToCharacter(String value) {
        if (null == value || 0 == value.length()) {
            return null;
        }
        return new Character(value.charAt(0));
    }
}
