/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.jdbc.binding;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.tranql.ql.QueryBinding;

/**
 *
 *
 *
 * @version $Revision: 155 $ $Date: 2004-12-02 18:12:10 -0500 (Thu, 02 Dec 2004) $
 */
public class LongBinding extends AbstractBinding {
    public LongBinding(int index, QueryBinding transform) {
        super(index, transform);
    }

    public Object getValue(ResultSet rs) throws SQLException {
        long value = rs.getLong(index);
        return rs.wasNull() ? null : new Long(value);
    }

    public void setValue(PreparedStatement ps, Object value) throws SQLException {
        if (value == null) {
            ps.setNull(index, getSQLType());
        } else {
            ps.setLong(index, ((Long) value).longValue());
        }
    }

    public void register(CallableStatement cs) throws SQLException {
        cs.registerOutParameter(index, getSQLType());
    }

    public Object getValue(CallableStatement cs) throws SQLException {
        long value = cs.getLong(index);
        return cs.wasNull() ? null : new Long(value);
    }
    
    public int getSQLType() {
        return Types.BIGINT;
    }
}
