/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.jdbc.binding;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.tranql.field.FieldTransform;
import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;
import org.tranql.ql.SchemaBinding;
import org.tranql.schema.Attribute;
import org.tranql.schema.Entity;
import org.tranql.schema.FKAttribute;
import org.tranql.sql.TypeConverter;
import org.tranql.sql.jdbc.InputBinding;
import org.tranql.sql.jdbc.OutputBinding;
import org.tranql.sql.jdbc.ResultBinding;
import org.tranql.sql.jdbc.SQLBinding;

/**
 *
 * @version $Revision: 155 $ $Date: 2004-12-02 18:12:10 -0500 (Thu, 02 Dec 2004) $
 */
public class TypeConverterBinding implements ResultBinding, InputBinding, OutputBinding {
    private final FieldTransform transform;
    private final InputBinding inputBinding;
    private final OutputBinding outputBinding;
    private final ResultBinding resultBinding;
    private final SQLBinding sqlBinding;
    private final SchemaBinding schemaBinding;
    private TypeConverter typeConverter;

    public TypeConverterBinding(FieldTransform transform, InputBinding inputBinding, TypeConverter typeConverter) {
        this.transform = transform;
        this.inputBinding = inputBinding; 
        this.sqlBinding = inputBinding;
        this.schemaBinding = inputBinding;
        this.typeConverter = typeConverter;
        outputBinding = null;
        resultBinding = null;
    }

    public TypeConverterBinding(FieldTransform transform, OutputBinding outputBinding, TypeConverter typeConverter) {
        this.transform = transform;
        this.outputBinding = outputBinding;
        this.sqlBinding = outputBinding;
        this.schemaBinding = outputBinding;
        this.typeConverter = typeConverter;
        inputBinding = null;
        resultBinding = null;
    }

    public TypeConverterBinding(FieldTransform transform, ResultBinding resultBinding, TypeConverter typeConverter) {
        this.transform = transform;
        this.resultBinding = resultBinding;
        this.sqlBinding = resultBinding;
        this.schemaBinding = resultBinding;
        this.typeConverter = typeConverter;
        inputBinding = null;
        outputBinding = null;
    }

    public Object getValue(ResultSet rs) throws SQLException {
        return typeConverter.convertSQLToJavaType(resultBinding.getValue(rs));
    }

    public void getValue(ResultSet rs, Row row) throws SQLException, FieldTransformException {
        transform.set(row, getValue(rs));
    }

    public void setValue(PreparedStatement ps, Row row) throws SQLException, FieldTransformException {
        setValue(ps, transform.get(row));
    }
    
    public void setValue(PreparedStatement ps, Object value) throws SQLException {
        inputBinding.setValue(ps, typeConverter.convertJavaToSQLType(value));
    }
    
    public void register(CallableStatement cs) throws SQLException {
        outputBinding.register(cs);
    }

    public Object getValue(CallableStatement cs) throws SQLException {
        return typeConverter.convertSQLToJavaType(outputBinding.getValue(cs));
    }

    public int getSQLType() {
        return sqlBinding.getSQLType();
    }
    
    public TypeConverter getConverter() {
        return typeConverter;
    }

    public void setEntity(Entity entity) {
        schemaBinding.setEntity(entity);
    }

    public Entity getEntity() {
        return schemaBinding.getEntity();
    }

    public boolean isAttributeBinding() {
        return schemaBinding.isAttributeBinding();
    }

    public void setAttribute(Attribute attribute) {
        schemaBinding.setAttribute(attribute);
    }

    public Attribute getAttribute() {
        return schemaBinding.getAttribute();
    }

    public boolean isFKAttributeBinding() {
        return schemaBinding.isFKAttributeBinding();
    }

    public void setFKAttribute(FKAttribute fkAttribute) {
        schemaBinding.setFKAttribute(fkAttribute);
    }

    public FKAttribute getFKAttribute() {
        return schemaBinding.getFKAttribute();
    }

    public boolean isUnknownBinding() {
        return schemaBinding.isUnknownBinding();
    }
}
