/**
 * 
 */
package org.tranql.sql.prefetch;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;

import org.tranql.cache.AlreadyAssociatedException;
import org.tranql.cache.CacheRow;
import org.tranql.cache.EmptySlotLoader;
import org.tranql.cache.InTxCache;
import org.tranql.field.FieldTransformException;
import org.tranql.field.Row;
import org.tranql.identity.GlobalIdentity;
import org.tranql.identity.IdentityDefiner;
import org.tranql.identity.UndefinedIdentityException;
import org.tranql.ql.QueryException;
import org.tranql.query.ResultHandler;
import org.tranql.sql.prefetch.PrefetchGroupHandler.CacheContext;

class ChildHandler implements ResultHandler, Serializable {
    private final IdentityDefiner idDefiner;
    private final EmptySlotLoader[] slotLoaders;
    private final ParentAccessor parentAccessor;
    private final ChildAccessor childAccessor;
    private final Collection childHandlers;

    public ChildHandler(IdentityDefiner idDefiner,
            EmptySlotLoader[] slotLoaders,
            ParentAccessor parentAccessor,
            ChildAccessor childAccessor,
            Collection childHandlers) {
        this.idDefiner = idDefiner;
        this.slotLoaders = slotLoaders;
        this.parentAccessor = parentAccessor;
        this.childAccessor = childAccessor;
        this.childHandlers = childHandlers;
    }
    
    public Object fetched(Row row, Object arg) throws QueryException {
        CacheContext ctx = (CacheContext) arg;
        CacheRow parentRow = ctx.cacheRow;
        InTxCache cache = ctx.cache;
        CacheRow childRow;
        GlobalIdentity id;
        try {
            id = idDefiner.defineIdentity(row);
            if (null == id) {
                return arg;
            }
            
            childRow = cache.get(id);
            if (null == childRow) {
                childRow = id.getTable().emptyRow(id);
            }
            for (int i=0; i < slotLoaders.length; i++) {
                slotLoaders[i].set(childRow, row);
            }
            
            parentAccessor.addGlobalIdentity(ctx, parentRow, id);
            childAccessor.setGlobalIdentity(childRow, parentRow.getId());
        } catch (FieldTransformException e) {
            throw new QueryException(e);
        } catch (UndefinedIdentityException e) {
            throw new QueryException(e);
        }
        
        ctx.cacheRow = childRow;
        for (Iterator iter = childHandlers.iterator(); iter.hasNext();) {
            ResultHandler handler = (ResultHandler) iter.next();
            handler.fetched(row, ctx);
        }
        ctx.cacheRow = parentRow;
        
        if (null == cache.get(id)) {
            try {
                cache.associate(childRow);
            } catch (AlreadyAssociatedException e) {
                throw new QueryException(e);
            }
        }

        return arg;
    }
    public Object endFetched(Object arg) throws QueryException {
        throw new UnsupportedOperationException();
    }
}