/**
 *  Copyright 2006 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.prefetch;

import org.tranql.cache.InTxCache;
import org.tranql.field.Row;
import org.tranql.ql.Query;
import org.tranql.ql.QueryException;
import org.tranql.query.QueryCommand;
import org.tranql.query.ResultHandler;

/**
 *
 *
 * @version $Revision: 340 $ $Date: 2006-06-03 18:13:16 -0400 (Sat, 03 Jun 2006) $
 */
public class PrefetchQueryCommand implements QueryCommand {
    private final QueryCommand command;
    private final PrefetchGroupHandler groupHandler;
    
    public PrefetchQueryCommand(QueryCommand command, PrefetchGroupHandler groupHandler) {
        this.command = command;
        this.groupHandler = groupHandler;
    }

    public Query getQuery() {
        return command.getQuery();
    }

    public Object execute(InTxCache cache, ResultHandler handler, Row params, Object arg) throws QueryException {
        PrefetchGroupHandler newHandler = new PrefetchGroupHandler(groupHandler, handler);
        Object result = newHandler.execute(cache, command, params, arg);
        return result;
    }
    
    public QueryCommand getQueryCommand() {
        return command;
    }
    
    public String toString() {
        return "PrefetchQueryCommand [" + command + "]";
    }
}
