/**
 *
 *  Copyright 2004 Jeremy Boynes
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.tranql.sql.sql92;

import java.util.ArrayList;
import java.util.List;

import org.tranql.ql.AttributeReference;
import org.tranql.ql.BinaryOperation;
import org.tranql.ql.From;
import org.tranql.ql.Node;
import org.tranql.ql.ParameterReference;
import org.tranql.ql.Query;
import org.tranql.ql.QueryBinding;
import org.tranql.ql.QueryBindingImpl;
import org.tranql.ql.QueryException;
import org.tranql.ql.QuerySource;
import org.tranql.ql.Select;
import org.tranql.ql.Where;
import org.tranql.sql.Column;
import org.tranql.sql.Table;

/**
 * 
 * 
 * @version $Revision: 155 $ $Date: 2004-12-02 18:12:10 -0500 (Thu, 02 Dec 2004) $
 */
public class SQL92QueryBuilder {
    private final SQL92Schema schema;

    public SQL92QueryBuilder(SQL92Schema schema) {
        this.schema = schema;
    }

    public Query buildSelectAllById(String tableName, boolean forUpdate) throws QueryException {
        Table table = schema.getTable(tableName);
        if (table == null) {
            throw new IllegalArgumentException("Schema does not contain Table: "+ tableName);
        }

        QuerySource querySource = new QuerySource(table, "T", forUpdate);
        
        Select select = new Select(forUpdate);
        Node expr = null;
        List columns = table.getAttributes();
        List paramTransforms = new ArrayList();
        List resultTransforms = new ArrayList(columns.size());
        int param = 0;
        for (int i=0; i < columns.size(); i++) {
            Column column = (Column) columns.get(i);
            select.addChild(new AttributeReference(querySource, column));
            resultTransforms.add(new QueryBindingImpl(i, table, column));
            if (column.isIdentity()) {
                paramTransforms.add(new QueryBindingImpl(param, table, column));

                Node term = new BinaryOperation(BinaryOperation.EQ).addChild(new AttributeReference(querySource, column)).addChild(new ParameterReference(param));
                expr = (expr == null) ? term : new BinaryOperation(BinaryOperation.AND).addChild(expr).addChild(term);
                param++;
            }
        }
        Node from = new From().addChild(querySource);
        Node where = new Where().addChild(expr);
        Query query = new Query((QueryBinding[])paramTransforms.toArray(new QueryBinding[paramTransforms.size()]), (QueryBinding[])resultTransforms.toArray(new QueryBinding[resultTransforms.size()]));
        query.addChild(select).addChild(from).addChild(where);
        return query;
    }
}
